/**
 * classe reprsentant une urne
 */
 
public class UrneModel {
 int nbTirage;                 // nombre de boules par tirage
 int nbBleu,nbRouge,nbVert;    // nombre de boules de chaque couleur
 int nbTotal;                  // nombre total de boules
 int[] tabTirage= new int[27]; // rsultats des tirages
 double[] tabProba= new double[27];
 boolean[] tabChoix= new boolean[27];
 int tirages;                  // nombre de tirages effectus
 boolean remise;               // type de tirage

 public UrneModel() {
  nbTirage=3;
  remise=false;
  defUrne(10,8,5);
 }

 public void defUrne(int b, int r, int v) {
  nbBleu=b;
  nbRouge=r;
  nbVert=v;
  nbTotal=b+r+v;
  initTirages();
  initProbas();
  resetChoix();
 }
  
 double frequence(int n) {
  if (tirages==0) return 0.0;
  else return (1.0*tabTirage[n])/tirages;
 }

 public void initTirages() {
  for (int i=0; i<27; i++) tabTirage[i]=0;
  tirages=0;
 }

 int hasard(int n) {
  return (int)Math.floor(Math.random()*n);
 }
 
 String donneTirage() {
  //effectue un tirage
  int n;
  int nb,nr,nv,nt;
  String S;
  S="";
  nb=nbBleu; nr=nbRouge; nv=nbVert; nt=nbTotal;
  for (int i=0; i<nbTirage; i++) {
   n=hasard(nt);
   if (n<nb) {
    S=S+"B";
    if (!remise) nb--;
   }
   else if (n<nb+nr) {
    S=S+"R";
    if (!remise) nr--;
   }
   else {
    S=S+"V";
    if (!remise) nv--;
   }
   if (!remise) nt--;
  }
  return S;
 }

 int numTirage(String S) {
  // renvoie l'indice du tirage dans le tableau
  int rep=1;
  for (int i=0; i<S.length();i++) {
   switch (S.charAt(i)) {
    case 'B': 
     rep=3*rep-2;
     break;
    case 'R': 
     rep=3*rep-1;
     break;
    case 'V': 
     rep=3*rep;
   }  
  } 
  return rep-1;
 }

 public void faireTirage() {
  tabTirage[numTirage(donneTirage())]++;
  tirages++;
 }
 
 public void faireTirage(int n) {
  for (int i=0; i<n; i++) faireTirage();
 }
 
 public String defTirage(int n) {
  //renvoie le tirage correspondant  un indice du tableau
  String S="";
  int q,r;
  q=n;
  for (int i=0; i<nbTirage; i++) {
   r=q % 3;
   switch (r) {
    case 0 : 
     S="B"+S;
     break;
    case 1 : 
     S="R"+S;
     break;
    case 2 : 
     S="V"+S;
   }
   q=q/3;
  }
  return S;
 }

 public void initProbas() {
  for (int i=0; i<27; i++) tabProba[i]=proba(i);
 }
 
 public double proba(int n) {
  double r=1;
  String S;
  int nb,nv,nr,nt;
  nb=nbBleu; nr=nbRouge; nv=nbVert; nt=nbTotal;
  S=defTirage(n);
  for (int i=0; i<S.length(); i++) {
   switch (S.charAt(i)) {
    case 'B' :
     r=r*nb/nt;
     if (!remise) nb--;
     break;
    case 'R' :
     r=r*nr/nt;
     if (!remise) nr--;
     break;
    case 'V' :
     r=r*nv/nt;
     if (!remise) nv--;
   }
   if (!remise) nt--;
  }
  return r;
 } 
 
 public void resetChoix() {
  //dtermine les branches  afficher
  for (int i=0; i<3; i++) tabChoix[i]=true;
  for (int i=3; i<9; i++) 
   if (nbTirage>=2) tabChoix[i]=true;
   else tabChoix[i]=false;
  for (int i=9; i<27; i++) 
   if (nbTirage>=3) tabChoix[i]=true;
   else tabChoix[i]=false;
 }
 
}

