/*
 * Copyright (c) 2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * Fractal.java
 * JDK 1.1 
 * Version 1.1 Rvision 30/12/1999
 * Applet permettant d'afficher des fractales du type Courbe de Koch 
 * dont les paramtres sont dans un fichier.
 * Paramtres du tag Applet :
 * NAME fichier VALUE "nom_du_fichier"
 *
 */

import java.util.*;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import jkb.geom.*;


public class fractal extends Applet 
 implements ActionListener {
 // l'applet contient une zone de dessin et une zone  boutons de
 // commandes
 
 FractalView zoneDessin;
 
 public void init() {
  Button bouton;
  zoneDessin=new FractalView();
  setBackground(Color.darkGray);
  setLayout(new BorderLayout());
  add("South",doButtonPanel());
  add("Center",zoneDessin);
 }
 
 public void start() {
  String nomf;
  nomf=getParameter("fichier");
  if (nomf!=null && nomf.length()!=0) {
   readFile(nomf);
  }
  else {
   nomf=getParameter("script");
   if (nomf!=null && nomf.length()!=0) {
    zoneDessin.execStr(nomf);
   }
   else zoneDessin.defKoch();
  } 
  zoneDessin.fenetreOrthoCentre(getSize().width/3);
 }
 
 public void execScript(String S) {
  zoneDessin.ordre=0;
  zoneDessin.messageErreur="";
  try {
   zoneDessin.execStr(S);
  }
  catch (Exception e) {
   zoneDessin.messageErreur=e.toString();
  }
  zoneDessin.repaint();   
 }
 
 Panel doButtonPanel() {
  Button bouton;
  Panel pan=new Panel();
  pan.setLayout(new FlowLayout());
  bouton=new Button("Dpart");
  bouton.addActionListener(this);
  pan.add(bouton);
  bouton=new Button("Suivant");
  bouton.addActionListener(this);
  pan.add(bouton);
  bouton=new Button("Prcdent");
  bouton.addActionListener(this);
  pan.add(bouton);
  pan.setBackground(Color.lightGray);
  return pan;
 }
 
 public void readFile(String S) {
  zoneDessin.ordre=0;
  zoneDessin.messageErreur="";
  try {
   URL source = new URL(getCodeBase(), S);
   BufferedReader br = new BufferedReader
    (new InputStreamReader(source.openStream()));
   zoneDessin.readObject(br); 
  }
  catch(Exception e) {
   zoneDessin.messageErreur=e.toString();
  }
  zoneDessin.repaint();
 }
 
 public void actionPerformed(ActionEvent evt) {
  if (evt.getActionCommand().equals("Suivant")) {
   zoneDessin.suivant();
  }
  else if (evt.getActionCommand().equals("Prcdent")) {
   zoneDessin.precedent();
  }
  else if (evt.getActionCommand().equals("Dpart")) {
   zoneDessin.ordre=0;
   zoneDessin.repaint();
  } 
 }
 
 public Insets getInsets() {
  int Bord=2;
  return new Insets(Bord,Bord,Bord,Bord);
 }
 
}

class FractalView extends Plan {
 
 Vector xBase,yBase;
 Vector xGen,yGen;
 int ordre;
 int pointsBase;
 int pointsGen;
 String nomCourbe;
 String messageErreur;
 int maxIter=10;

 FractalView() {
  xBase=null; yBase=null;
  xGen=null; yGen=null;
  ordre=0;
  messageErreur="";
  couleurFond = new Color(0xFFFFFF);
 }

 void defKoch() {
 //donnes par dfaut
  nomCourbe="Koch";
  pointsBase=3;
  xBase=new Vector(pointsBase+1);
  yBase=new Vector(pointsBase+1);
  
  xBase.addElement(new Double(1.0));   
  xBase.addElement(new Double(-0.5)); 
  xBase.addElement(new Double(-0.5)); 
  xBase.addElement(new Double(1.0));  
  
  yBase.addElement(new Double(0.0));
  yBase.addElement(new Double(0.866025));
  yBase.addElement(new Double(-0.866025));
  yBase.addElement(new Double(0.0));
  
  pointsGen=5;
  xGen=new Vector(pointsGen);
  yGen=new Vector(pointsGen);
  
  xGen.addElement(new Double(0.0));
  xGen.addElement(new Double(0.333333));
  xGen.addElement(new Double(0.5));
  xGen.addElement(new Double(0.666666));
  xGen.addElement(new Double(1.0));
  
  yGen.addElement(new Double(0.0));
  yGen.addElement(new Double(0.0));
  yGen.addElement(new Double(-0.2887));
  yGen.addElement(new Double(0.0));
  yGen.addElement(new Double(0.0));
 }

 void execStr(String S) {
  StringTokenizer st=new StringTokenizer(S);
  nomCourbe=st.nextToken();
  pointsBase=Integer.parseInt(st.nextToken());
  xBase=new Vector(pointsBase+1);
  yBase=new Vector(pointsBase+1);
  for (int i=0; i<pointsBase; i++) {
   xBase.addElement(new Double(st.nextToken()));
   yBase.addElement(new Double(st.nextToken()));
  }
  xBase.addElement(new Double(getVal(xBase,0)));
  yBase.addElement(new Double(getVal(yBase,0))); 
  pointsGen=Integer.parseInt(st.nextToken());
  xGen=new Vector(pointsGen);
  yGen=new Vector(pointsGen);
  for (int i=0; i<pointsGen; i++) {
   xGen.addElement(new Double(st.nextToken()));
   yGen.addElement(new Double(st.nextToken()));
  } 
 }
 
 void readObject(Reader is) throws IOException {
  int i,t;
  StreamTokenizer stream = new StreamTokenizer(is);
  stream.eolIsSignificant(false);
  stream.commentChar('#');
  nomCourbe="";
  do {
   t=stream.nextToken();
   if (t==StreamTokenizer.TT_WORD) nomCourbe=nomCourbe+" "+stream.sval;
  } while (t==StreamTokenizer.TT_WORD); 
  pointsBase=(int)stream.nval;
  xBase=new Vector(pointsBase+1);
  yBase=new Vector(pointsBase+1);
  for (i=0; i<pointsBase; i++) {
   t=stream.nextToken();
   xBase.addElement(new Double(stream.nval));
   t=stream.nextToken();
   yBase.addElement(new Double(stream.nval));
  }
  xBase.addElement(new Double(getVal(xBase,0)));
  yBase.addElement(new Double(getVal(yBase,0))); 
  t=stream.nextToken();
  pointsGen=(int)stream.nval;
  xGen=new Vector(pointsGen);
  yGen=new Vector(pointsGen);
  for (i=0; i<pointsGen; i++) {
   t=stream.nextToken();
   xGen.addElement(new Double(stream.nval));
   t=stream.nextToken();
   yGen.addElement(new Double(stream.nval));
  } 
 }

 void suivant() {
  if (ordre<maxIter) {
   ordre++;
   repaint();
  } 
 }
 
 void precedent() {
  if (ordre>0) {
   ordre--;
   repaint();
  } 
 }

 double getVal(Vector v, int n) {
  return ((Double)v.elementAt(n)).doubleValue(); 
 }
 
 public void paint(Graphics g) {
  // procedure d'affichage
  int i;
  super.paint(g);
  g.setColor(Color.black);
  g.drawString(nomCourbe,10,10);
  if (messageErreur.length()>0) {
   g.drawString(messageErreur,10,40);
  }
  else {
   for (i=0; i<pointsBase; i++) {
    dessineFractale(g, getVal(xBase,i),getVal(yBase,i),
     getVal(xBase,i+1),getVal(yBase,i+1),ordre);
   }  
  } 
 }
 
 synchronized void dessineFractale(Graphics g, double X1, double Y1, double X2, double Y2, int N) {
  Vector SX,SY;
  int i;
  double d=Math.sqrt((X2-X1)*(X2-X1)+(Y2-Y1)*(Y2-Y1));
  if ((N>0) && (d>=unPixelX())) {
   SX=new Vector(pointsGen);
   SY=new Vector(pointsGen);
   for (i=0; i<pointsGen; i++) {
    SX.addElement(new Double((X2-X1)*getVal(xGen,i)-(Y2-Y1)*getVal(yGen,i)+X1));
    SY.addElement(new Double((Y2-Y1)*getVal(xGen,i)+(X2-X1)*getVal(yGen,i)+Y1));
   }
   for (i=0; i<pointsGen-1; i++) {
    dessineFractale(g,getVal(SX,i),getVal(SY,i),getVal(SX,i+1),getVal(SY,i+1),N-1);
    //SX.removeAllElements();
    //SY.removeAllElements();
   }
  }    
  else {
   traceVers(g,X1,Y1,0);
   traceVers(g,X2,Y2,1);
  }
 }
 

}

