/*
 * Copyright (c) 2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * MathStr.java
 * JDK 1.1  
 * Version 1.0 Rvision 15/06/2000
 * Partie du package calkul interprtant une chane de caractres
 * en tant que formule mathmatique.
 *
 */

package jkb.calkul;

import java.util.Hashtable;
import java.util.Vector;
import jkb.utils.ComStr;
import jkb.utils.DicoObj;

/**
 * Classe du package calkul permettant d'valuer une
 * une chane de caractres.
 */
public class MathStr extends ComStr {

 /**
  * Constantes codes d'erreurs.
  */
 public final static int errPar=1;       // erreur de parenthses 
 public final static int errSynt=2;      // erreur de syntaxe 
 public final static int errNum=3;       // nombre mal crit
 public final static int errIdent=4;     // identificateur inconnu
 public final static int errNoCalc=5;    // pas de calcul  faire
 public final static int errParaFonc=6;  // parenthse aprs fonction
 public final static int errNonMath=7;   // caractre non valuable
 public final static int errFin=8;       // fin prmature
 public final static int errCode=9;      // code inconnu
 public final static int maxErrMathStr=9;

 public static String mathCars="+-*/^(),"+'\"';
 public static String espaceCars=" "+'\n'+'\r';
 
 /**
  * Constructeur de l'objet; attend la chane  valuer
  * en paramtre.
  */
 public MathStr(String S) {
  super(S,mathCars+espaceCars);
 } 

 /**
  * Constructeur de l'objet; attend la chane  valuer
  * en paramtre.
  */
 public MathStr(String S, String sep) {
  super(S,mathCars+espaceCars+sep);
 } 

 /**
  * Fonction renvoyant un message d'erreur avec entre
  * parenthses la position de l'erreur.
  */
 public String messageErreur() {
  String rep="";
  switch (codeErreur) {
   case errPar :
    rep="Parenthse manquante."+posErreur();
    break;
   case errSynt :
    rep="Erreur de syntaxe."+posErreur();
    break;
   case errNum :
    rep="Nombre mal crit."+posErreur();
    break;
   case errIdent :
    rep="Identificateur inconnu."+posErreur();
    break;
   case errNoCalc :
    rep="Expression attendue."+posErreur();
    break;
   case errParaFonc :
    rep="Erreur fonction."+posErreur();
    break;
   case errNonMath :
    rep="Caractre non valuable."+posErreur();
    break;
   case errCode :
    rep="Code non trait."+posErreur();
    break;
   case errFin :
    rep="Oprateur attendu."+posErreur();
    break;
   default :
    rep=super.messageErreur();
  }
  return rep;
 }
 
 void lireParams(MathNode o, Hashtable h) {
  // lire parenthse ouvrante
  lireSuivant();
  if (suivant=='(') {
   // premier argument
   lireSuivant();
   passeEspaces();
   o.ajouteBranche(evalExp(h,false));
   passeEspaces();
   // arguments suivants
   while (suivant==',' && codeErreur==0) {
    lireSuivant();
    passeEspaces();
    o.ajouteBranche(evalExp(h,false));
    passeEspaces();
   }
   // lire parenthse fermante
   if (suivant==')' && codeErreur==0) lireSuivant();
   else marqueErreur(errParaFonc);
  }
  else marqueErreur(errParaFonc);
 }
 
 public MathNode traiteIdent(DicoObj o, Hashtable h) {
  marqueErreur(errCode);
  return null;
 }
 
 MathNode evalFirstItem(Hashtable h) {
 // valuation d'exprressions commenant par + ou -
  MathNode rep=null;
  if (codeErreur!=0) return null;
  if (suivant=='-') rep=new MathNode(MathNode.tOpp,null);
  else if (suivant=='+') rep=new MathNode(MathNode.tId,null);
  if (rep!=null) {
   lireSuivant();
   passeEspaces();
   MathNode dat=evalFacteur(h,false);
   rep.ajouteBranche(dat);
  } 
  else rep=evalItem(h); 
  return rep;
 }

 MathNode evalItem(Hashtable h) {
 // valuation d'un lment minimal : nombre, variable, formule, fonction
  MathNode rep=null;
  DicoObj DO=null;
  String temp;
  if (codeErreur!=0) return null;
  passeEspaces();
  if (((suivant>='0') && (suivant<='9')) || (suivant=='.')) { 
   //lecture d'un nombre 
   try {
    Double D=new Double(motCourant);
    rep=new MathNode(MathNode.tNombre,D);
    lireSuivant();
   } catch (NumberFormatException e) { 
    marqueErreur(errNum);
   } 
  }
  else if ((suivant>='A' && suivant<='Z') || (suivant>='a' && suivant<='z')) {
   //identificateur : variable, fonction ou formule
   if (h==null) DO=null;
   else DO=(DicoObj)h.get(motCourant.toUpperCase());
   if (DO==null) marqueErreur(errIdent);
   else {
    if (DO.oType==MathNode.tNombre) {
     rep=(MathNode)DO;
     lireSuivant();
    } 
    else if (DO.oType==MathNode.tMathNode) {
     rep=(MathNode)DO.obj;
     lireSuivant();
    } 
    else if (DO.oType>MathNode.tFonc && DO.oType<MathNode.tMaxFonc) {
     rep=new MathNode(DO.oType,null);
     lireParams(rep,h);
    }
    else rep=traiteIdent(DO,h);
   } 
  }
  else if (suivant=='(') {
   //parenthse ouvrante
   lireSuivant();
   passeEspaces();
   rep=evalExp(h,false);
   passeEspaces();
   if (suivant!=')') marqueErreur(errPar);
   else lireSuivant(); 
  }
  else if (suivant==carFin) marqueErreur(errNoCalc);
  else if (suivant=='+' || suivant=='-' || suivant=='*'
   || suivant=='/' || suivant=='^') marqueErreur(errPar);
  else marqueErreur(errNonMath);
  return rep;
 }

 MathNode evalFacteur(Hashtable h, boolean first) {
  //calcul des puissances
  MathNode rep=null;
  MathNode dat;
  if (codeErreur!=0) return null;
  if (first) dat=evalFirstItem(h);
  else dat=evalItem(h);
  passeEspaces();
  if (suivant=='^') {
   rep=new MathNode(MathNode.tPuiss,null);
   rep.ajouteBranche(dat);
   while ((codeErreur==0) && (suivant=='^')) {
    lireSuivant();
    dat=evalItem(h);
    rep.ajouteBranche(dat);
   }
  } 
  else rep=dat;
  return rep;
 }

 MathNode evalTerm(Hashtable h, boolean first) {
  //calcul des quotients et des produits
  MathNode rep=null;
  MathNode dat,tmp;
  char c;
  dat=evalFacteur(h,first);
  passeEspaces();
  if (suivant=='*' || suivant=='/') {
   rep=new MathNode(MathNode.tProduit,null);
   rep.ajouteBranche(dat);
   while ((codeErreur==0) && ((suivant=='*') || (suivant=='/'))) {
    c=suivant;  
    lireSuivant();
    dat=evalFacteur(h,false);
    if (codeErreur==0) {
     if (c=='/') {
      tmp=new MathNode(MathNode.tInv,null);
      tmp.ajouteBranche(dat);
      rep.ajouteBranche(tmp);
     } 
     else rep.ajouteBranche(dat);
    }
   } 
  }
  else rep=dat;
  return rep;
 }

 /**
  * Fonction permettant de transformer une chane de caractres
  * en objet MathNode valuable;
  * - h est une table des mots utilisables (variables et fonctions)
  * - tout indique si une vrification de fin de chane doit
  * tre effectue
  */
 public MathNode evalExp(Hashtable h, boolean tout) {
  //calcul des additions et soustractions
  MathNode rep=null;
  MathNode dat,tmp;
  char c;
  dat=evalTerm(h,true);
  if (suivant=='+' || suivant=='-') {
   rep=new MathNode(MathNode.tSomme,null);
   rep.ajouteBranche(dat);
   while ((codeErreur==0) && ((suivant=='+') || (suivant=='-'))) {
    c=suivant;
    lireSuivant();
    dat=evalTerm(h,false);
    if (c=='+') rep.ajouteBranche(dat);
    else {
     tmp=new MathNode(MathNode.tOpp,null);
     tmp.ajouteBranche(dat);
      rep.ajouteBranche(tmp);
    }
   }
  } 
  else rep=dat;
  /*  ce point de l'valuation le caractre suivant doit tre
        1. un marqueur de fin de ligne (&)
        2. une parenthse droite ou une virgule
        3. le dbut d'un oprateur boolen = ou < ou >      
  */       
  passeEspaces();
  if ((tout) && (suivant!=carFin)) marqueErreur(errFin);
  if (codeErreur!=0) return null;
  else return rep;
 }


}

