/*
 * Copyright (c) 2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * MathObj.java
 * JDK 1.1  
 * Version 1.0 Rvision 15/06/2000
 * Partie du package calkul dfinissant des arbres de calcul.
 *
 */

package jkb.calkul;

import java.util.Vector;
import java.util.Hashtable;
import jkb.utils.DicoObj;

/**
 * Classe du package calkul permettant de grer les
 * variables et les fonctions.
 */
public class MathNode extends DicoObj {

 /**
  * Constantes codes d'objets.
  */
 public final static int tMathNode=10;          // base
 public final static int tNombre=tMathNode+1;   // nombre
 public final static int tSomme=tMathNode+2;    // somme de plusieurs termes
 public final static int tProduit=tMathNode+3;  // produit de plusieurs facteurs
 public final static int tPuiss=tMathNode+4;    // puissance
 
 public final static int tEgal=tMathNode+5;
 public final static int tDiff=tMathNode+6;
 public final static int tSup=tMathNode+7;
 public final static int tInf=tMathNode+8;
 public final static int tSupEgal=tMathNode+9;
 public final static int tInfEgal=tMathNode+10;
 
 public final static int tFonc=tMathNode+20;    // fonctions
 public final static int tAbs=tFonc+1;    // valeur absolue
 public final static int tAcos=tFonc+2;   // arc cosinus
 public final static int tAsin=tFonc+3;   // arc sinus
 public final static int tAtan=tFonc+4;   // arc tangente
 public final static int tCos=tFonc+5;    // cosinus
 public final static int tEnt=tFonc+6;    // partie entire
 public final static int tExp=tFonc+7;    // exponentielle
 public final static int tHasard=tFonc+8; // nombre au hasard
 public final static int tLn=tFonc+9;     // logarithme
 public final static int tRac=tFonc+10;   // racine carre
 public final static int tSin=tFonc+11;   // sinus
 public final static int tTan=tFonc+12;   // tangente
 public final static int tOpp=tFonc+13;   // oppos
 public final static int tInv=tFonc+14;   // inverse
 public final static int tId=tFonc+15;    // identit
 public final static int tMax=tFonc+16;   // maximum
 public final static int tMin=tFonc+17;   //minimum
 
 public final static int tMaxFonc=tFonc+18;
 
 
 public final static int mRadian=0;
 public final static int mDegre=1;
 
 /**
  * Mode de calcul pour les angles : mRadian ou mDegre
  */
 public static int modeAngle=mRadian;
 
 /**
  * Constructeur gnrique
  */
 public MathNode(int c, Object o) {
  super(c,o);
 }
 
 /**
  * Constructeur pour les valeurs numriques
  */
 public MathNode(double d) {
  super(tNombre,new Double(d));
 }

 /**
  * Ajouter une branche
  */
 public void ajouteBranche(MathNode o) {
  if (oType!=tNombre) {
   if (obj==null) obj=new Vector(5,5);
  branches().addElement(o);
  }
 }
 
 /**
  * Accs aux branches
  */
 public Vector branches() {
  if (oType!=tNombre) return (Vector)obj;
  else return null;
 }
 
 /**
  * Renvoie un lment d'une branche
  */
 public MathNode feuille(int n) {
  if (branches()!=null) return (MathNode)branches().elementAt(n);
  else return null;
 }
 
 /**
  * Calcul de la valeur
  */
 public double donneValeur() {
  double val=Double.NaN;
  if (oType==tNombre) {
   val=((Double)obj).doubleValue();
  }
  else if (oType>=tSomme && oType<=tPuiss) {
   // oprations
   val=feuille(0).donneValeur();
   int i=1;
   while (i<branches().size()) {
    switch (oType) {
     case tSomme :
      val=val+feuille(i).donneValeur();
      break;
     case tProduit :
      val=val*feuille(i).donneValeur();
      break;
     case tPuiss :
      val=Math.pow(val,feuille(i).donneValeur());
      break;
    }
    i++;
   }
  }
  else if (oType>=tEgal && oType<=tInfEgal) {
   // comparaisons
   switch (oType) {
    case tEgal :
     if (feuille(0).donneValeur()==feuille(1).donneValeur()) val=1;
     else val=0;
     break;
    case tDiff :
     if (feuille(0).donneValeur()!=feuille(1).donneValeur()) val=1;
     else val=0;
     break;
    case tSup :
     if (feuille(0).donneValeur()>feuille(1).donneValeur()) val=1;
     else val=0;
     break;
    case tInf :
     if (feuille(0).donneValeur()<feuille(1).donneValeur()) val=1;
     else val=0;
     break;
    case tSupEgal :
     if (feuille(0).donneValeur()>=feuille(1).donneValeur()) val=1;
     else val=0;
     break;
    case tInfEgal :
     if (feuille(0).donneValeur()<=feuille(1).donneValeur()) val=1;
     else val=0;
     break;
   }
  } 
  else if (oType>tFonc && oType<tMaxFonc) {
   // fonctions
   switch (oType) {
     case tAbs :
      val=Math.abs(feuille(0).donneValeur());
      break; 
     case tAcos :  
      val=Math.acos(feuille(0).donneValeur());
      if (modeAngle==mDegre) val=val*180/Math.PI;
      break; 
     case tAsin : 
      val=Math.asin(feuille(0).donneValeur());
      if (modeAngle==mDegre) val=val*180/Math.PI;
      break;
     case tAtan : 
      val=Math.atan(feuille(0).donneValeur());
      if (modeAngle==mDegre) val=val*180/Math.PI;
      break;
     case tCos : 
      if (modeAngle==mRadian) val=Math.cos(feuille(0).donneValeur());
      else val=Math.cos(feuille(0).donneValeur()*Math.PI/180.0);
      break;
     case tEnt : 
      val=Math.floor(feuille(0).donneValeur());
      break;
     case tExp : 
      val=Math.exp(feuille(0).donneValeur());
      break;
     case tHasard : 
      val=Math.random()*feuille(0).donneValeur();
      break;
     case tLn :
      val=Math.log(feuille(0).donneValeur());
      break;
     case tRac :
      val=Math.sqrt(feuille(0).donneValeur());
      break;
     case tSin : 
      if (modeAngle==mRadian) val=Math.sin(feuille(0).donneValeur());
      else val=Math.sin(feuille(0).donneValeur()*Math.PI/180.0);
      break;
     case tTan : 
      if (modeAngle==mRadian) val=Math.tan(feuille(0).donneValeur());
      else val=Math.tan(feuille(0).donneValeur()*Math.PI/180.0);
      break;
     case tOpp :
      val=-feuille(0).donneValeur();
      break;
     case tInv :
      val=1.0/feuille(0).donneValeur();
      break;
     case tId :
      val=feuille(0).donneValeur();
      break;
     case tMax :
      val=Math.max(feuille(0).donneValeur(),feuille(1).donneValeur());
      break;
   }  
  } 
  return val;
 }


 /**
  * Mise  jour d'un dictionnaire de base pour l'valuation
  * des formules.
  */
 public static void initDico(Hashtable h) {
  h.put("PI",new MathNode(Math.PI));
  h.put("ABS",new DicoObj(tAbs,null));
  h.put("ACOS",new DicoObj(tAcos,null));
  h.put("ASIN",new DicoObj(tAsin,null));
  h.put("ATAN",new DicoObj(tAtan,null));
  h.put("COS",new DicoObj(tCos,null));
  h.put("ENT",new DicoObj(tEnt,null));
  h.put("EXP",new DicoObj(tExp,null));
  h.put("HASARD",new DicoObj(tHasard,null));
  h.put("LN",new DicoObj(tLn,null));
  h.put("MAX",new DicoObj(tMax,null));
  h.put("MIN",new DicoObj(tMin,null));
  h.put("RAC",new DicoObj(tRac,null));
  h.put("SIN",new DicoObj(tSin,null));
  h.put("TAN",new DicoObj(tTan,null));
 }
 
 /**
  * Changement de valeur numrique pour une variable
  */
 public static void defVar(String nom, double valeur, Hashtable dico) {
  DicoObj DO=(DicoObj)dico.get(nom.toUpperCase());
  if (DO==null) dico.put(nom.toUpperCase(),new MathNode(valeur));
  else if (DO.oType==tNombre) DO.obj=new Double(valeur);
 }

 /**
  * Changement de formule pour une variable
  */
 public static void defForm(String nom, MathNode form, Hashtable dico) {
  DicoObj DO=(DicoObj)dico.get(nom.toUpperCase());
  if (DO==null) dico.put(nom.toUpperCase(),new DicoObj(tMathNode,form));
  else if (DO.oType==tMathNode) DO.obj=form;
 }

}


