package jkb.geo3d;

import java.util.Vector;
import java.awt.*;

public class Face extends Obj3d {
    Vector sommets = new Vector(10,10);

    public Face() {
	super(Obj3d.iFace);
    }

    public Face(Point3d P1, Point3d P2, Point3d P3) {
	super(Obj3d.iFace);
	sommets.addElement(P1);
	sommets.addElement(P2);
	sommets.addElement(P3);
	nom="["+P1.nom+P2.nom+P3.nom+"]";
    }

    public Face(Point3d P1, Point3d P2, Point3d P3, Point3d P4) {
	super(Obj3d.iFace);
	sommets.addElement(P1);
	sommets.addElement(P2);
	sommets.addElement(P3);
	sommets.addElement(P4);
	nom="["+P1.nom+P2.nom+P3.nom+P4.nom+"]";
    }

    public Face(Vector V) {
	super(Obj3d.iFace);
	for (int i=0; i<V.size()-1; i++) sommets.addElement(V.elementAt(i));
    }

    public boolean coincide(Obj3d o) {
	boolean rep=false;
	// faire
	return rep;
    }

    public Point3d donneSommet(int num) {
	Point3d P=null;
	if (num>=0 && num<sommets.size()) P=(Point3d) sommets.elementAt(num);
	return P;
    }

    public void ajouteSommet(Point3d P) {
	sommets.addElement(P);
    }

    public int nombreSommets() {
	return sommets.size();
    }

    public Segment3d donneArete(int num) {
	Segment3d S=null;
	if (num==sommets.size()-1) S=new Segment3d(donneSommet(num), donneSommet(0));
	else if (num>=0 && num<sommets.size()) 
	    S=new Segment3d(donneSommet(num), donneSommet(num+1));
	return S;
    }

    public Point3d pointSur(String n, double x1, double x2) {
	Point3d P0, P1, P2, rep;
	Point3d u,v;
	P0=donneSommet(0);
	P1=donneSommet(1);
	P2=donneSommet(sommets.size()-1);
	u=Point3d.vecteur(P0,P1);
	v=Point3d.vecteur(P0,P2);
	rep=new Point3d(n,P0.x+x1*u.x+x2*v.x,P0.y+x1*u.y+x2*v.y,P0.z+x1*u.z+x2*v.z);
	//rep.type=Obj3d.iPointSur;
	return rep;
    }

    public Point3d normale() {
	Point3d P0,P1,P2;
	Point3d u,v,w;
	P0=donneSommet(0);
	P1=donneSommet(1);
	P2=donneSommet(nombreSommets()-1);
	u=Point3d.vecteur(P0,P1);
	v=Point3d.vecteur(P0,P2);
	return Point3d.prodVect(u,v);
    }

    public double equation(Point3d P) {
	//calcule ax+by+cz+d pour P
	Point3d u,v;
	u=Point3d.vecteur(donneSommet(0),P);
	v=normale();
	return Point3d.prodScal(u,v);
    }

    public double distPoint(Point3d P) {
	Point3d u,v;
	u=Point3d.vecteur(donneSommet(0),P);
	v=normale();
	return (Point3d.prodScal(u,v))/v.norme();
    }

    public boolean estSommet(Point3d P) {
	boolean rep=false;
	int i=0;
	while (!rep && i<sommets.size()) {
	    if (donneSommet(i)==P) rep=true;
	    else i++;
	}
	return rep;
    }

    public static Face plan(String nom, Point3d A, Point3d B, Point3d C) {
	Face rep=new Face(A,B,C);
	rep.nom=nom;
	rep.type=Obj3d.iPlan;
	rep.visible=false;
	return rep;
    }

    public Polygon fairePolygone(Repere R) {
	int[] x,y;
	int[] c; 
	Point3d P;
	int nbSommets=sommets.size();
	x=new int[nbSommets];
	y=new int[nbSommets];
	for (int i=0; i<nbSommets; i++) {
	    P=donneSommet(i);
	    c=R.coordEcran(P.x,P.y,P.z);
	    x[i]=c[0]; y[i]=c[1];
	}
	return new Polygon(x,y,nbSommets);
    }

    public boolean cache(Point3d P, Repere R) {
	boolean rep=false;
	Point3d obs;
	Polygon poly;
	//Segment3d arete;
	double d0=distPoint(P);
	double d=equation(P);
	int[] c;
	// le point est-il sur un ct de la face ?
	//boolean ok=false;
	//int i=0;
	//while (!ok && i<sommets.size()) {
	//    arete=donneArete(i);
	//    if (arete.contient(P)) ok=true;
	//    i++;
	//}
	//if (!ok) {
	if (Math.abs(d0)>1000*Point3d.zero) {
	    obs=new Point3d(R.xObs,R.yObs,R.zObs);
	    if (equation(obs)*d<0) {
		poly=fairePolygone(R);
		c=R.coordEcran(P.x,P.y,P.z);
		rep=poly.contains(c[0],c[1]);
	    }
	}
	return rep;
    }

    public static Point3d intersection(Segment3d S, Face f, Repere R) {
	Point3d rep;
	double t,den;
	Polygon poly;
	Segment3d arete;
	int[] c;
	Point3d U=Point3d.vecteur(S.P1,S.P2);
	Point3d V=f.normale();
	den=Point3d.prodScal(U,V);
	if (Math.abs(den)<Point3d.zero) rep=new Point3d(stInterParallele);
	else {
	    t=f.equation(S.P1)/den;
	    rep=new Point3d(S.P1.x-t*U.x,S.P1.y-t*U.y,S.P1.z-t*U.z);
	    if (!S.contient(rep)) rep.statut=stHorsSegment;
	    else if (f.type==Obj3d.iFace) {
		// le point est-il sur un ct de la face ?
		boolean ok=false;
		int i=0;
		while (!ok && i<f.sommets.size()) {
		    arete=f.donneArete(i);
		    if (arete.contient(rep)) ok=true;
		    i++;
		}
		if (!ok) {
		    //le point est-il  l'intrieur de la face ?
		    poly=f.fairePolygone(R);
		    c=R.coordEcran(rep.x,rep.y,rep.z);
		    if (!poly.contains(c[0],c[1])) rep.statut=stHorsFace;
		}
	    }
	}
	return rep;
    }
}
