package jkb.geo3d;

import java.net.*;
import java.io.*;
import java.util.*;

public class Fichier3d {

    //constantes pour types de fichiers
    public static int F3D=0;
    public static int OFF=1;
    public static int OBJ=2;

    int type=F3D;
    URL source;
    int erreur=0;

    public Fichier3d(int letype, URL lasource) {
	type=letype;
	source=lasource;
    }

    public String chargeFichier() {
	String S = "";
	try {
	    String ligne = "";
	    BufferedReader br = new BufferedReader(new InputStreamReader(source.openStream()));
	    while (null != (ligne = br.readLine()))
		if (!ligne.startsWith("#")) S=S+ligne+"\n";
	    br.close();         
	}
	catch(Exception e) {
	    erreur=1;
	}
	return S;
    }

    public Vector chargeLignes() {
	Vector V=new Vector();
	try {
	    String ligne = "";
	    BufferedReader br = new BufferedReader(new InputStreamReader(source.openStream()));
	    while (null != (ligne = br.readLine()))
		if (!ligne.startsWith("#")) V.addElement(ligne);
	    br.close();         
	}
	catch(Exception e) {
	    V=null;
	}
	return V;
    }

    public void traiteFichier(Fig3d f) {
	if (type==F3D) {
	    String S=chargeFichier();
	    Script3d src=new Script3d(S);
	    src.compile(f);
	}
	else if (type==OFF) 
	    traiteFichierOff(f);
	else if (type==OBJ)
	    traiteFichierObj(f);
    }

    public void lirePointOff(String S, Fig3d f) {
	Point3d P=new Point3d(0,0,0);
	StringTokenizer st=new StringTokenizer(S," ",false);
	String tmp;
	Double D;
	int i=0;
	while (st.hasMoreTokens() && i<3) {
	    tmp=st.nextToken();
	    D=new Double(tmp);
	    if (i==0) P.x=D.doubleValue();
	    else if (i==1) P.y=D.doubleValue();
	    else P.z=D.doubleValue();
	    i++;
	}
	f.points.addElement(P);
    }

    public void lireFaceOff(String S, Fig3d f) {
	Face f0=new Face();
	Point3d P;
	StringTokenizer st=new StringTokenizer(S," ",false);
	String tmp;
	Integer N;
	tmp=st.nextToken();
	N=new Integer(tmp);
	int nbPoints=N.intValue();
	for (int i=0; i<nbPoints; i++) {
	    tmp=st.nextToken();
	    N=new Integer(tmp);
	    P=(Point3d)f.points.elementAt(N.intValue());
	    f0.sommets.addElement(P);
	}
	f.faces.addElement(f0);
    }

    public void traiteFichierOff(Fig3d f) {
	Vector V=chargeLignes();
	if (V==null) return;
	String tmp;
	Integer N;
	int nbPoints=0;
	int nbFaces=0;
	tmp=(String)V.elementAt(0);
	StringTokenizer st=new StringTokenizer(tmp," ",false);
	N=new Integer(st.nextToken());
	nbPoints=N.intValue();
	N=new Integer(st.nextToken());
	nbFaces=N.intValue();
	for (int i=0; i<nbPoints; i++) {
	    tmp=(String)V.elementAt(i+1);
	    lirePointOff(tmp, f);
	}
	for (int i=0; i<nbFaces; i++) {
	    tmp=(String)V.elementAt(i+1+nbPoints);
	    lireFaceOff(tmp,f);
	}
    }

    public void lirePointObj(String S, Fig3d f) {
	Point3d P=new Point3d(0,0,0);
	StringTokenizer st=new StringTokenizer(S," ",false);
	String tmp;
	Double D;
	int i=0;
	tmp=st.nextToken(); //lire "v "
	while (st.hasMoreTokens() && i<3) {
	    tmp=st.nextToken();
	    D=new Double(tmp);
	    if (i==0) P.x=D.doubleValue();
	    else if (i==1) P.y=D.doubleValue();
	    else P.z=D.doubleValue();
	    i++;
	}
	f.points.addElement(P);
    }

    public void lireFaceObj(String S, Fig3d f) {
	Face f0=new Face();
	Point3d P;
	StringTokenizer st=new StringTokenizer(S," ",false);
	String tmp;
	Integer N;
	tmp=st.nextToken(); //lire "f "
	while (st.hasMoreTokens()) {
	    tmp=st.nextToken();
	    N=new Integer(tmp);
	    P=(Point3d)f.points.elementAt(N.intValue()-1);
	    f0.sommets.addElement(P);
	}
	f.faces.addElement(f0);
    }

    public void lireSegObj(String S, Fig3d f) {
	Point3d P1,P2;
	StringTokenizer st=new StringTokenizer(S," ",false);
	String tmp;
	Integer N;
	tmp=st.nextToken(); //lire "l "
	tmp=st.nextToken(); // premier point
	N=new Integer(tmp);
	P1=(Point3d)f.points.elementAt(N.intValue()-1);
	while (st.hasMoreTokens()) { //points suivants
	    tmp=st.nextToken();
	    N=new Integer(tmp);
	    P2=(Point3d)f.points.elementAt(N.intValue()-1);
	    f.segments.addElement(new Segment3d(P1,P2));
	    P1=P2;
	}
    }

    public void traiteFichierObj(Fig3d f) {
	Vector V=chargeLignes();
	if (V==null) return;
	String tmp;
	for (int i=0; i<V.size(); i++) {
	    tmp=(String)V.elementAt(i);
	    if (tmp.startsWith("v ")) lirePointObj(tmp,f);
	    else if (tmp.startsWith("f ")) lireFaceObj(tmp,f);
	    else if (tmp.startsWith("l ")) lireSegObj(tmp,f);
	}
    }
}
