package jkb.geo3d;

/* Classe reprsentant un point ou un vecteur
   de l'espace.
   Chaque reprsentant est dfini par ses coordonnes x,y,z.
*/

import java.util.Vector;

public class Point3d extends Obj3d {
    public double x,y,z;
    public boolean marque=false;
    public static double zero=0.00000001;

    public Point3d(int code) {
	super(Obj3d.iPoint);
	marqueErreur(code);
    }

    public Point3d(double a, double b, double c) {
	super(Obj3d.iPoint);
	x=a; y=b; z=c;
    }

    public Point3d(String n, double a, double b, double c) {
	super(Obj3d.iPoint);
	x=a; y=b; z=c;
	nom=n;
    }

    public Point3d(Vector V) {
	super(Obj3d.iPoint);
	Double d;
	d=(Double)V.elementAt(0);
	x=d.doubleValue();
	d=(Double)V.elementAt(1);
	y=d.doubleValue();
	d=(Double)V.elementAt(2);
	z=d.doubleValue();
    }

    public double donneX() {
	return x;
    }

    public double donneY() {
	return y;
    }

    public double donneZ() {
	return z;
    }

    public double norme() {
	return Math.sqrt(x*x+y*y+z*z);
    }

    public boolean estNul() {
	return (norme()<zero);
    }

    public boolean coincide(Obj3d o) {
	boolean rep=false;
	Point3d P;
	if (o.estPoint()) {
	    P=(Point3d)o;
	    if ((x-P.x)*(x-P.x)+(y-P.y)*(y-P.y)+(z-P.z)*(z-P.z)<zero)
		rep=true;
	}
	return rep;
    }

    public Point3d translate(Point3d A, Point3d B) {
	return new Point3d(x+B.x-A.x, y+B.y-A.y, z+B.z-A.z);
    }

    // mthodes statiques pour faciliter les calculs

    public static boolean egalZero(double r) {
	return (Math.abs(r)<zero);
    }

    public static Point3d vecteur(Point3d P1, Point3d P2) {
	return new Point3d(P2.x-P1.x,P2.y-P1.y,P2.z-P1.z);
    }

    public static double prodScal(Point3d P1, Point3d P2) {
	return P1.x*P2.x+P1.y*P2.y+P1.z*P2.z;
    }

    public static Point3d prodVect(Point3d P1, Point3d P2) {
	return new Point3d(P1.y*P2.z-P1.z*P2.y, P1.z*P2.x-P1.x*P2.z, P1.x*P2.y-P1.y*P2.x);
    }

    public static double distance(Point3d P1, Point3d P2) {
	return vecteur(P1,P2).norme();
    }

    public static Point3d milieu(Point3d P1, Point3d P2) {
	return new Point3d((P1.x+P2.x)/2,(P1.y+P2.y)/2,(P1.z+P2.z)/2);
    }

    public static Point3d isoBarycentre(Vector v) {
	Point3d P,G;
	G=new Point3d(0,0,0);
	for (int i=0; i<v.size(); i++) {
	    P=(Point3d)v.elementAt(i);
	    G.x=G.x+P.x; G.y=G.y+P.y; G.z=G.z+P.z;
	}
	G.x=G.x/v.size();
	G.y=G.y/v.size();
	G.z=G.z/v.size();
	return G;
    }

    public static boolean alignement(Point3d P1, Point3d P2, Point3d P3) {
	Point3d u,v,w;
	u=vecteur(P1,P2);
	v=vecteur(P1,P3);
	w=prodVect(u,v);
	return w.estNul();
    }

    public static boolean coplanaire(Point3d P1, Point3d P2, Point3d P3, Point3d P4) {
	Point3d u,v,w,n;
	u=vecteur(P1,P2);
	v=vecteur(P1,P3);
	w=vecteur(P1,P4);
	n=prodVect(u,v);
	return (Math.abs(prodScal(n,w))<zero);
    }

}
