package jkb.geo3d;

import java.util.Vector;
import jkb.utils.ComStr;

public class Script3d extends ComStr {

    /**
     * codes des erreurs d'interprtation
     */
    public static final int errMotInconnu=1;
    public static final int errNomDouble=2;
    public static final int errParam=3;
    public static final int errParenth=4;
    public static final int errPolygone=5;
    public static final int errCouleur=6;
    public static final int errObjet=7;
    public static final int errCommande=8;

    public static final int oCommande=1000;
    public static final int oCube=oCommande+1;
    public static final int oTetra=oCommande+2;
    public static final int oCoinCube=oCommande+3;
    public final static int oOcta=oCommande+4;
    public final static int oPyramide=oCommande+5;
    public final static int oPrisme=oCommande+6;
    public final static int oPavedroit=oCommande+7;
    public static final int oMarque=oCommande+100;
    public static final int oCache=oCommande+101;

 
    // constructeur
    public Script3d(String S) {
	super(S," (),="+'\n'+'\r');
    }

    // excution du script dans la figure fig
    public void compile(Fig3d fig) {
	//interprtation de la chaine
	while (suivant!=carFin && codeErreur==0) {
	    execInstruction(fig);
	}
    }

    // codes des mots du langage
    public static int codeMot(String S) {
	//code d'une instruction
	if (S==null) return 0;
	else if (S.equals("POINT")) return Obj3d.iPoint;
	else if (S.equals("SEGMENT")) return Obj3d.iSegment;
	else if (S.equals("DROITE")) return Obj3d.iDroite;
	else if (S.equals("POINTSUR")) return Obj3d.iPointSur;
	else if (S.equals("INTERSECTION")) return Obj3d.iIntersection;
	else if (S.equals("PARALLELE")) return Obj3d.iParallele;
	else if (S.equals("FACE")) return Obj3d.iFace;
	else if (S.equals("PLAN")) return Obj3d.iPlan;
	else if (S.equals("CUBE")) return oCube;
	else if (S.equals("TETRAEDRE")) return oTetra;
	else if (S.equals("COINCUBE")) return oCoinCube;
	else if (S.equals("OCTAEDRE")) return oOcta;
	else if (S.equals("PYRAMIDE")) return oPyramide;
	else if (S.equals("PRISME")) return oPrisme;
	else if (S.equals("PAVE")) return oPavedroit;
	else if (S.equals("MARQUER")) return oMarque;
	else if (S.equals("CACHER")) return oCache;
	else return 0;
    }

    // construction des objets 
    public Obj3d faireObj(Fig3d fig, int commande, Vector v) {
	// construction d'un objet
	Obj3d obj=null;
	Obj3d o1,o2;
	Segment3d S1, S2;
	Point3d P;
	Face F;
	Double d1,d2;
	switch (commande) {
	case Obj3d.iPoint :
	    obj=new Point3d(v);
	    break;
	case Obj3d.iDroite :
	    obj=new Segment3d(v);
	    obj.type=Obj3d.iDroite;
	    break;
	case Obj3d.iSegment :
	    obj=new Segment3d(v);
	    break;
	case Obj3d.iFace :
	    obj=new Face(v);
	    break;
	case Obj3d.iPlan :
	    obj=new Face(v);
	    obj.type=Obj3d.iPlan;
	    obj.visible=false;
	    break;
	case Obj3d.iPointSur :
	    if (v.size()==3) { 
		S1=(Segment3d)v.elementAt(0);
		d1=(Double)v.elementAt(1);
		P=S1.pointSur("",d1.doubleValue());
		obj=P;
	    }
	    else if (v.size()==4) {
		F=(Face)v.elementAt(0);
		d1=(Double)v.elementAt(1);
		d2=(Double)v.elementAt(2);
		P=F.pointSur("",d1.doubleValue(),d2.doubleValue());
		obj=P;
	    }
	    break;
	case Obj3d.iIntersection :
	    o1=(Obj3d)v.elementAt(0);
	    o2=(Obj3d)v.elementAt(1);
	    if (o1.estSegment() && o2.estSegment()) {
		S1=(Segment3d)o1; S2=(Segment3d)o2;
		P=Segment3d.intersection(S1,S2);
		obj=P;
	    }
	    else if (o1.estSegment() && o2.estFace()) {
		S1=(Segment3d)o1; F=(Face)o2;
		P=Face.intersection(S1,F,fig.R);
		obj=P;
	    }
	    break;
	case Obj3d.iParallele :
	    S1=(Segment3d)v.elementAt(0);
	    P=(Point3d)v.elementAt(1);
	    S2=Segment3d.parallele("",S1,P);
	    obj=S2;
	    break;
	default : 
	    obj=new Obj3d(Obj3d.stErreur); 
	}
	return obj;
    } 

    public Vector lireParams(Fig3d fig) {
	// lecture des paramtres entre parenthses
	Vector rep=new Vector(10);
	String chaine="";
	Double d=null;
	Obj3d obj;
	boolean fini=false;
	//y a-t-il des paramtres ?
	if (suivant=='(') {
	    //passer la parenthse
	    lireSuivant();
	    passeEspaces();
	    while (!fini && codeErreur==0) {
		// est-ce un nombre ?
		try {
		    d=new Double(motCourant);
		} catch (NumberFormatException e) {
		    d=null;
		} 
		if (d!=null) {
		    rep.addElement(d);
		    chaine=chaine+"N";
		    lireSuivant();
		    passeEspaces();
		}
		else {
		    //est-ce un objet dj cr ?
		    obj=fig.trouveObj(motCourant.toUpperCase());
		    if (obj!=null) {
			lireSuivant();
			passeEspaces();
		    }
		    else {
			//est-ce un objet  crer
			int c=codeMot(motCourant.toUpperCase());
			if (c==0 || c>=oCommande) marqueErreur(errParam);
			else {
			    //passer la commande
			    lireSuivant();
			    passeEspaces();
			    //lire les paramtres
			    Vector V=lireParams(fig);
			    //crer le nouvel objet
			    if (codeErreur==0) obj=faireObj(fig,c,V);
			    if (obj!=null) {
				//obj.visible=false;
				//fig.ajouteObj(obj);
			    }
			} 
		    }
		    if (obj!=null) {
			//ajouter l'objet
			rep.addElement(obj);
			switch (obj.type) {
			case Obj3d.iPoint :
			case Obj3d.iPointSur :
			case Obj3d.iIntersection : 
			    chaine=chaine+"P";
			    break;
			case Obj3d.iSegment :
			case Obj3d.iDroite :
			case Obj3d.iParallele :
			    chaine=chaine+"S";
			    break;
			case Obj3d.iFace :
			case Obj3d.iPlan :
			    chaine=chaine+"F";
			    break;
			default : 
			    chaine=chaine+"I"; 
			}
		    }
		} 
		//lire sparateur
		if (suivant!=',' && suivant!=')') marqueErreur(errParenth); 
		else {
		    if (suivant==')') fini=true;
		    // lire mot suivant
		    lireSuivant();
		    passeEspaces();
		}
	    }
	}
	if (codeErreur==0) rep.addElement(chaine); // ajouter la chane de dfinition
	else rep=null;
	return rep;
    }
 
    public void execInstruction(Fig3d fig) {
	int commande;
	String leNom, str1;
	Obj3d obj;
	//rcuprer le nom de l'objet  crer
	str1=motCourant.toUpperCase();
	lireSuivant();
	passeEspaces();
	if (suivant=='=') {
	    leNom=str1;
	    //passer le signe =
	    lireSuivant();
	    passeEspaces();
	    str1=motCourant.toUpperCase();
	    lireSuivant();
	    passeEspaces();
	}
	else leNom="";
	// dtermination de la commande
	commande=codeMot(str1);
	if (commande==0) marqueErreur(errMotInconnu);
	else {
	    Vector v=lireParams(fig);
	    if (commande>oCommande && codeErreur==0) {
		execCommande(commande,leNom,v,fig);
	    } 
	    else if (codeErreur==0) {
		obj=faireObj(fig,commande,v);
		if (obj!=null) {
		    obj.nom=leNom;
		    fig.ajouteObj(obj);
		} 
	    } 
	} 
    } 

    void execCommande(int com, String n, Vector v, Fig3d fig) {
	//excution des commandes
	Double d,e,f;
	Segment3d S;
	Point3d P;
	Obj3d obj;
	switch (com) {
	case oCube :
	    d=(Double)v.elementAt(0);
	    Solide.ajouteCube(n,d.doubleValue(),fig);
	    break;
	case oPavedroit :
	    d=(Double)v.elementAt(0);
	    e=(Double)v.elementAt(1);
	    f=(Double)v.elementAt(2);
	    Solide.ajoutePavedroit(n,d.doubleValue(),e.doubleValue(),f.doubleValue(),fig);
	    break;
	case oTetra :
	    d=(Double)v.elementAt(0);
	    Solide.ajouteTetraEdre(n,d.doubleValue(),fig);
	    break;
	case oCoinCube :
	    d=(Double)v.elementAt(0);
	    Solide.ajouteCoinCube(n,d.doubleValue(),fig);
	    break;
	case oOcta :
	    d=(Double)v.elementAt(0);
	    Solide.ajouteOctaEdre(n,d.doubleValue(),fig);
	    break;
	case oPyramide :
	    d=(Double)v.elementAt(0);
	    e=(Double)v.elementAt(1);
	    Solide.ajoutePyramide(n,d.doubleValue(),e.doubleValue(),fig);
	    break;
	case oPrisme :
	    d=(Double)v.elementAt(0);
	    e=(Double)v.elementAt(1);
	    Solide.ajoutePrisme(n,d.doubleValue(),e.doubleValue(),fig);
	    break;
	case oMarque :
	    for (int i=0; i<v.size()-1; i++) {
		P=(Point3d)v.elementAt(i);
		P.marque=true;
	    }
	    break;
	case oCache :
	    for (int i=0; i<v.size()-1; i++) {
		obj=(Obj3d)v.elementAt(i);
		obj.visible=false;
	    }
	    break;
	}
    } 
 
    public String messageErreur() {
	String S="";
	switch (codeErreur) {
	case errMotInconnu :
	    S="Mot inconnu."+posErreur();
	    break;
	case errNomDouble :
	    S="Nom dj utilis."+posErreur();
	    break;
	case errParam :
	    S="Paramtre incorrect."+posErreur();
	    break;
	case errParenth :
	    S="Parenthse attendue."+posErreur();
	    break;
	case errPolygone :
	    S="Polygone mal dfini."+posErreur();
	    break;
	case errCouleur :
	    S="Mauvais nom de couleur."+posErreur();
	    break;
	case errObjet :
	    S="Code objet non gr."+posErreur();
	    break;
	case errCommande :
	    S="Code commande non gr."+posErreur();
	    break;
	default :
	    S=super.messageErreur();
	}
	return S;
    }
 
}

