package jkb.geo3d;

import java.util.Vector;
import jkb.utils.*;

// classe reprsentant les segments et les droites

public class Segment3d extends Obj3d {
    Point3d P1,P2;

    public Segment3d(String n, Point3d A, Point3d B) {
	super(Obj3d.iSegment);
	P1=A; P2=B; nom=n; 
    }

    public Segment3d(Point3d A, Point3d B) {
	super(Obj3d.iSegment);
	P1=A; P2=B; nom="["+A.nom+B.nom+"]"; 
    }

    public Segment3d(Vector V) {
	super(Obj3d.iSegment);
	P1=(Point3d)V.elementAt(0); 
	P2=(Point3d)V.elementAt(1); 
    }

    public double coeffAlignement(Point3d M) {
	Point3d u,v;
	double k;
	k=100000;
	u=Point3d.vecteur(P1,P2);
	v=Point3d.vecteur(P1,M);
	if (Math.abs(u.x)>Point3d.zero) k=v.x/u.x; 
	else if (Math.abs(u.y)>Point3d.zero) k=v.y/u.y; 
	else if (Math.abs(u.z)>Point3d.zero) k=v.z/u.z;
	return k;
    }

    public boolean contient(Point3d M) {
	Point3d u,v;
	double k;
	boolean rep=false;
	if (P1.coincide(M) || P2.coincide(M)) rep=true; 
	else if (Point3d.alignement(P1,P2,M)) {
	    if (type==Obj3d.iDroite) rep=true;
	    else {
		k=-1;
		u=Point3d.vecteur(P1,P2);
		v=Point3d.vecteur(P1,M);
		if (Math.abs(u.x)>Point3d.zero) k=v.x/u.x; 
		else if (Math.abs(u.y)>Point3d.zero) k=v.y/u.y; 
		else if (Math.abs(u.z)>Point3d.zero) k=v.z/u.z;
		if (k>=0 && k<=1) rep=true;
	    }
	}
	return rep;
    }

    public boolean coincide(Obj3d o) {
	boolean rep=false;
	Segment3d S;
	if (type==Obj3d.iSegment && o.type==Obj3d.iSegment) {
	    S=(Segment3d)o;
	    if ((P1.coincide(S.P1) && P2.coincide(S.P2)) || 
		(P1.coincide(S.P2) && P2.coincide(S.P1)))  rep=true;
	}
	if (type==Obj3d.iDroite && o.type==Obj3d.iDroite) {
	    S=(Segment3d)o;
	    if (Point3d.alignement(P1,P2,S.P1) && Point3d.alignement(P1,P2,S.P2))
		rep=true;
	}
	return rep;
    }

    // pour construire un point sur un segment
    public Point3d pointSur(String nom, double k) {
	Point3d u=Point3d.vecteur(P1,P2);
	Point3d P=new Point3d(nom,P1.x+k*u.x,P1.y+k*u.y,P1.z+k*u.z);
	P.marque=true;
	//P.type=Obj3d.iPointSur;
	return P;
    }

    public static Point3d intersection(Segment3d S1, Segment3d S2) {
	Point3d ab, dc, ac; 
	Point3d rep=null;
	double delta;
	double[] sol;
	if (Point3d.coplanaire(S1.P1,S1.P2,S2.P1,S2.P2)) {
	    ab=Point3d.vecteur(S1.P1,S1.P2);
	    dc=Point3d.vecteur(S2.P2,S2.P1);
	    ac=Point3d.vecteur(S1.P1,S2.P1);
	    if (!Point3d.prodVect(ab,dc).estNul()) {
		// les segments ne sont pas parallles
		sol=Div.syseq2x2(ab.x,dc.x,ac.x,ab.y,dc.y,ac.y);
		if (sol==null) sol=Div.syseq2x2(ab.y,dc.y,ac.y,ab.z,dc.z,ac.z);
		if (sol==null) sol=Div.syseq2x2(ab.x,dc.x,ac.x,ab.z,dc.z,ac.z);
		rep=new Point3d(S1.P1.x+sol[0]*ab.x,S1.P1.y+sol[0]*ab.y,S1.P1.z+sol[0]*ab.z);
		if (!S1.contient(rep) || !S2.contient(rep)) rep=new Point3d(Obj3d.stHorsSegment);
	    }
	    else rep=new Point3d(Obj3d.stInterParallele);
	}
	else rep=new Point3d(Obj3d.stInterNonCoplanaire);
	return rep;
    }

    // cration d'une droite
    public static Segment3d droite(String n, Point3d A, Point3d B) {
	Segment3d rep=new Segment3d(n,A,B);
	rep.type=Obj3d.iDroite;
	return rep;
    }

    public static Segment3d droite(Point3d A, Point3d B) {
	Segment3d rep=new Segment3d(A,B);
	rep.nom="("+A.nom+B.nom+")";
	rep.type=Obj3d.iDroite;
	return rep;
    }

    public static Segment3d parallele(String n, Segment3d S, Point3d A) {
	Point3d B=A.translate(S.P1,S.P2);
	Segment3d rep=droite(n,A,B);
	return rep;
    }
}
