package jkb.geo3d;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import jkb.calkul.*;
import jkb.utils.*;

public class Surface extends Espace implements Dessinateur {

    double xMin = -5;
    double xMax = 5;
    double yMin = -5;
    double yMax = 5;
    double zMin = -5;
    double zMax = 5;
    int modePlan = 0;
    double valPlan = 0;
    int nbLignes = 11;
    int nbPoints = 50;
    int mode = 0; //0 pour z=f(x,y) et 1 pour y=g(x,z) lignes de niveaux
    String defSurface = "";
    MathNode formule=null;
    Hashtable dico=null; 
    boolean erreurCalc;
    public int statut=0;

    public Surface() {
	R.fixeEchelle(8);
	D=this;
	dico = new Hashtable();
	MathNode.initDico(dico);
	definirSurface("z=rac(x^2+y^2)/2");
    }

    public void nombreLignes(int n) {
	nbLignes=n;
	repaint();
    }

    public void definirSurface(String S) {
	defSurface=S;
	formule=null;
	compile();
    }

    public void definirCube(String S) {
	Double[] tab = new Double[6];
	boolean ok=true;
	int n=0;
	StringTokenizer t=new StringTokenizer(S,",",false);
	while (ok && t.hasMoreTokens() && n<6) {
	    tab[n]=new Double(t.nextToken());
	    if (tab[n]==null) ok=false;
	    else n++;
	}
	if (ok && n==6) {
	    xMin=tab[0].doubleValue();
	    xMax=tab[1].doubleValue();
	    yMin=tab[2].doubleValue();
	    yMax=tab[3].doubleValue();
	    zMin=tab[4].doubleValue();
	    zMax=tab[5].doubleValue();
	    repaint();
	}
    }

    public void definirPlan(String S) {
	if (S.equals("")) {
	    modePlan=0; return;
	}
	StringTokenizer t=new StringTokenizer(S,"= ",false);
	String tmp;
	if (t.hasMoreTokens()) {
	    tmp=t.nextToken();
	    if (tmp.equals("x")) modePlan=1;
	    else if (tmp.equals("y")) modePlan=2;
	    else if (tmp.equals("z")) modePlan=3;
	}
	if (modePlan>0 && t.hasMoreTokens()) {
	    Double D=new Double(t.nextToken());
	    if (D==null) modePlan=0;
	    else valPlan=D.doubleValue();
	}
	else modePlan=0;
	if (modePlan==1 && valPlan>xMax) valPlan=xMax;
	else if (modePlan==2 && valPlan>yMax) valPlan=yMax;
	else if (modePlan==3 && valPlan>zMax) valPlan=zMax;
    }

    public void compile() {
	MathStr MS=new MathStr(defSurface.toUpperCase(),";=");
	if (MS.motCourant.equals("Y")) mode=1;
	else mode=0;
	MS.lireSuivant(); MS.passeEspaces(); //Lire Y ou Z
	MS.lireSuivant(); MS.passeEspaces(); //Lire =
	MathNode.defVar("X",0.0,dico);
	if (mode==0) MathNode.defVar("Y",0.0,dico);
	else MathNode.defVar("Z",0.0,dico);
	formule=MS.evalExp(dico,true);
	if (formule==null) statut=1; else statut=0;
    }

    public void boite(Graphics g) {
	g.setColor(couleurCache);
	traceVers(g,xMin,yMin,zMin,0);
	traceVers(g,xMax,yMin,zMin,1);
	traceVers(g,xMax,yMax,zMin,1);
	traceVers(g,xMin,yMax,zMin,1);
	traceVers(g,xMin,yMin,zMin,1);
	traceVers(g,xMin,yMin,zMax,0);
	traceVers(g,xMax,yMin,zMax,1);
	traceVers(g,xMax,yMax,zMax,1);
	traceVers(g,xMin,yMax,zMax,1);
	traceVers(g,xMin,yMin,zMax,1);
	traceVers(g,xMin,yMin,zMin,0);
	traceVers(g,xMin,yMin,zMax,1);
	traceVers(g,xMax,yMin,zMin,0);
	traceVers(g,xMax,yMin,zMax,1);
	traceVers(g,xMax,yMax,zMin,0);
	traceVers(g,xMax,yMax,zMax,1);
	traceVers(g,xMin,yMax,zMin,0);
	traceVers(g,xMin,yMax,zMax,1);
	traceVers(g,xMin,0,0,0);
	traceVers(g,xMax,0,0,1);
	traceVers(g,0,yMin,0,0);
	traceVers(g,0,yMax,0,1);
	traceVers(g,0,0,zMin,0);
	traceVers(g,0,0,zMax,1);
    }

    public double calcZ(double x, double y) {
	erreurCalc=false;
	MathNode.defVar("X",x,dico);
	MathNode.defVar("Y",y,dico);
	double z=formule.donneValeur();
	erreurCalc=Div.errCalc(z);
	if (erreurCalc) return 0.0; else return z;
    }

    public double calcY(double x, double z) {
	erreurCalc=false;
	MathNode.defVar("X",x,dico);
	MathNode.defVar("Z",z,dico);
	double y=formule.donneValeur();
	erreurCalc=Div.errCalc(z);
	if (erreurCalc) return 0.0; else return y;
    }

    public void defCouleur(Graphics g, double z) {
	double val=(zMax-z)*255.0/(zMax-zMin);
	int ival=(int)val;
	g.setColor(new Color(128,ival,0));
    }
 
    public void ligneX(Graphics g, double x, boolean colorier) {
	double y, z;
	double y0=0;
	double z0=0;
	double y1=0;
	double z1=0;
	double k;
	int drapeau=0; //nombre de points valides
	int dehors=1;  //hors cube
	double yCoupe=yMax;
	double zCoupe=zMax;
	if (modePlan==2) yCoupe=valPlan;
	if (modePlan==3) zCoupe=valPlan;
	for (int j=0; j<=nbPoints; j++) {
	    y=yMin+j*(yCoupe-yMin)/nbPoints;
	    z=calcZ(x,y);
	    if (!erreurCalc) {
		drapeau++;
		if (drapeau==1) {
		    y0=y; z0=z;
		    if (z>zCoupe || z<zMin) dehors=1;
		    else { // on est dans le cube
			dehors=0;
			traceVers(g,x,y,z,0);
		    }
		}
		else {
		    drapeau=2;
		    if (dehors==1) { // on est en dehors
			if (z>zCoupe || z<zMin) { // on reste en dehors
			    y0=y; z0=z;
			}
			else { // on entre dans le cube
			    if (z0>zCoupe) {
				k=(zCoupe-z)/(z0-z);
				z0=zCoupe;
			    }
			    else {
				k=(zMin-z)/(z0-z);
				z0=zMin;
			    }
			    y0=y+k*(y0-y);
			    traceVers(g,x,y0,z0,0);
			    if (modePlan==3 && Point3d.egalZero(z0-zCoupe)) {
				g.setColor(couleurDessin);
				grosPoint(g,2);
			    }
			    if (colorier) defCouleur(g,(z+z0)/2);
			    traceVers(g,x,y,z,1);
			    y0=y; z0=z;
			    dehors=0;
			}
		    } //fin dehors==1
		    else { //on est dans le cube
			y1=y; z1=z;
			if (z>zCoupe || z<zMin) { // on sort du cube
			    dehors=1;
			    if (z>zCoupe) {
				k=(zCoupe-z)/(z0-z);
				z=zCoupe;
			    }
			    else {
				k=(zMin-z)/(z0-z);
				z=zMin;
			    }
			    y=y+k*(y0-y);
			}
			if (colorier) defCouleur(g,(z+z0)/2);
			traceVers(g,x,y,z,1);
			if (modePlan==3 && Point3d.egalZero(z-zCoupe)) {
			    g.setColor(couleurDessin);
			    grosPoint(g,2);
			}
			y0=y1; z0=z1;
		    }
		}
	    } // fin !erreurCalc
	    else drapeau=0;
	} // for j
    }

    public void ligneY(Graphics g, double y, boolean colorier) {
	double x, z;
	double x0=0;
	double z0=0;
	double x1=0;
	double z1=0;
	double k;
	int drapeau=0; //nombre de points valides
	int dehors=1;  //hors cube
	double xCoupe=xMax;
	double zCoupe=zMax;
	if (modePlan==1) xCoupe=valPlan;
	if (modePlan==3) zCoupe=valPlan;
	for (int j=0; j<=nbPoints; j++) {
	    x=xMin+j*(xCoupe-xMin)/nbPoints;
	    z=calcZ(x,y);
	    if (!erreurCalc) {
		drapeau++;
		if (drapeau==1) {
		    x0=x; z0=z;
		    if (z>zCoupe || z<zMin) dehors=1;
		    else { // on est dans le cube
			dehors=0;
			traceVers(g,x,y,z,0);
		    }
		}
		else { // il y a dj un point
		    drapeau=2;
		    if (dehors==1) { // on est en dehors
			if (z>zCoupe || z<zMin) { // on reste en dehors
			    x0=x; z0=z;
			}
			else { // on entre dans le cube
			    if (z0>zCoupe) {
				k=(zCoupe-z)/(z0-z);
				z0=zCoupe;
			    }
			    else {
				k=(zMin-z)/(z0-z);
				z0=zMin;
			    }
			    x0=x+k*(x0-x);
			    traceVers(g,x0,y,z0,0);
			    if (modePlan==3 && Point3d.egalZero(z0-zCoupe)) {
				g.setColor(couleurDessin);
				grosPoint(g,2);
			    }
			    if (colorier) defCouleur(g,(z+z0)/2);
			    traceVers(g,x,y,z,1);
			    x0=x; z0=z;
			    dehors=0;
			}
		    } //fin dehors==1
		    else { //on est dans le cube
			x1=x; z1=z;
			if (z>zCoupe || z<zMin) { // on sort du cube
			    dehors=1;
			    if (z>zCoupe) {
				k=(zCoupe-z)/(z0-z);
				z=zCoupe;
			    }
			    else {
				k=(zMin-z)/(z0-z);
				z=zMin;
			    }
			    x=x+k*(x0-x);
			}
			if (colorier) defCouleur(g,(z+z0)/2); 
			traceVers(g,x,y,z,1);
			if (modePlan==3 && Point3d.egalZero(z-zCoupe)) {
			    g.setColor(couleurDessin);
			    grosPoint(g,2);
			}
			x0=x1; z0=z1;
		    }
		}
	    } // fin !erreurCalc
	    else drapeau=0;
	} // for j

    }

    public void ligneZ(Graphics g, double z) {
	double x, y;
	double x0=0;
	double y0=0;
	double x1=0;
	double y1=0;
	double k;
	int drapeau=0; //nombre de points valides
	int dehors=1;  //hors cube
	defCouleur(g,z);
	for (int j=0; j<=nbPoints; j++) {
	    x=xMin+j*(xMax-xMin)/nbPoints;
	    y=calcY(x,z);
	    if (!erreurCalc) {
		drapeau++;
		if (drapeau==1) {
		    x0=x; y0=y;
		    if (y>yMax || y<yMin) dehors=1;
		    else { // on est dans le cube
			dehors=0;
			traceVers(g,x,y,z,0);
		    }
		}
		else { // il y a dj un point
		    if (dehors==1) { // on est en dehors
			if (y>yMax || y<yMin) { // on reste en dehors
			    x0=x; y0=y;
			}
			else { // on entre dans le cube
			    if (y0>yMax) {
				k=(yMax-y)/(y0-y);
				y0=yMax;
			    }
			    else {
				k=(yMin-y)/(y0-y);
				y0=yMin;
			    }
			    x0=x+k*(x0-x);
			    traceVers(g,x0,y0,z,0);
			    traceVers(g,x,y,z,1);
			    x0=x; y0=y;
			    dehors=0;
			}
		    } //fin dehors==1
		    else { //on est dans le cube
			y1=y; x1=x;
			if (y>yMax || y<yMin) { // on sort du cube
			    dehors=1;
			    if (y>yMax) {
				k=(yMax-y)/(y0-y);
				y=yMax;
			    }
			    else {
				k=(yMin-y)/(y0-y);
				y=yMin;
			    }
			    x=x+k*(x0-x);
			}
			traceVers(g,x,y,z,1);
			x0=x1; y0=y1;
		    }
		}
	    } // fin !erreurCalc
	    else drapeau=0;
	} // for j

    }

    public void traceSurface(Graphics g) {
	int nbl=nbLignes;
	double lim=0;
	if (mode==0) {
	    if (modePlan==1) {
		nbl=(int)(nbl*(valPlan-xMin)/(xMax-xMin)); lim=valPlan;
	    }
 	    else {
		nbl=nbLignes; lim=xMax;
	    }
	    for (int i=0; i<=nbl; i++) ligneX(g, xMin+i*(lim-xMin)/nbl, true);
	    if (modePlan==2) {
		nbl=(int)(nbl*(valPlan-yMin)/(yMax-yMin)); lim=valPlan;
	    }
	    else {
		nbl=nbLignes; lim=yMax;
	    }
	    for (int i=0; i<=nbl; i++) ligneY(g, yMin+i*(lim-yMin)/nbl, true);
	    g.setColor(couleurDessin);
	    if (modePlan==1) {
		traceVers(g,valPlan,yMin,zMin,0);
		traceVers(g,valPlan,yMax,zMin,1);
		traceVers(g,valPlan,yMax,zMax,1);
		traceVers(g,valPlan,yMin,zMax,1);
		traceVers(g,valPlan,yMin,zMin,1);
		ligneX(g, valPlan, false);
	    }
	    else if (modePlan==2) {
		traceVers(g,xMin,valPlan,zMin,0);
		traceVers(g,xMax,valPlan,zMin,1);
		traceVers(g,xMax,valPlan,zMax,1);
		traceVers(g,xMin,valPlan,zMax,1);
		traceVers(g,xMin,valPlan,zMin,1);
		ligneY(g, valPlan, false);
	    }
	    else if (modePlan==3) {
		traceVers(g,xMin,yMin,valPlan,0);
		traceVers(g,xMax,yMin,valPlan,1);
		traceVers(g,xMax,yMax,valPlan,1);
		traceVers(g,xMin,yMax,valPlan,1);
		traceVers(g,xMin,yMin,valPlan,1);
	    }
	}
	else {
	    for (int i=0; i<=nbLignes; i++) ligneZ(g, zMin+i*(zMax-zMin)/nbLignes);
	}
    }

    public void dessineFig(Graphics g) {
	g.setColor(couleurFond);
	g.fillRect(0,0,R.largeur,R.hauteur);
	boite(g);
	if (formule!=null) traceSurface(g);
    }

}
