package jkb.geo3d;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import jkb.calkul.*;
import jkb.utils.*;

public class Surface extends Espace implements Dessinateur {

    double xMin = -5;
    double xMax = 5;
    double yMin = -5;
    double yMax = 5;
    double zMin = -5;
    double zMax = 5;
    int nbLignes = 11;
    int nbPoints = 50;
    int mode = 0; //0 pour z=f(x,y) et 1 pour y=g(x,z) lignes de niveaux
    String defSurface = "";
    Face planCoupe=null;
    int modePlan = 0;
    double valPlan = 0;
    MathNode formule=null;
    Hashtable dico=null; 
    boolean erreurCalc;
    public int statut=0;

    public Surface() {
	R.fixeEchelle(8);
	D=this;
	dico = new Hashtable();
	MathNode.initDico(dico);
	definirSurface("z=rac(x^2+y^2)/2");
    }

    public void nombreLignes(int n) {
	nbLignes=n;
	repaint();
    }

    public void definirSurface(String S) {
	defSurface=S;
	formule=null;
	compile();
    }

    public void definirPlan(String S) {
	Point3d P1, P2, P3, P4;
	StringTokenizer t=new StringTokenizer(S,"= ",false);
	if (t.hasMoreTokens()) {
	    if (t.nextToken().equals("x")) modePlan=1;
	    else if (t.nextToken().equals("y")) modePlan=2;
	    else if (t.nextToken().equals("z")) modePlan=3;
	}
	if (modePlan>0 && t.hasMoreTokens()) {
	    Double D=new Double(t.nextToken());
	    if (D==null) modePlan=0;
	    else valPlan=D.doubleValue();
	}
	else modePlan=0;
	if (modePlan==1) {
	    P1=new Point3d(valPlan,yMin,zMin);
	    P2=new Point3d(valPlan,yMax,zMin);
	    P3=new Point3d(valPlan,yMax,zMax);
	    P4=new Point3d(valPlan,yMin,zMax);
	    planCoupe=new Face(P1,P2,P3,P4);
	}
	else if (modePlan==2) {
	    P1=new Point3d(xMin,valPlan,zMin);
	    P2=new Point3d(xMax,valPlan,zMin);
	    P3=new Point3d(xMax,valPlan,zMax);
	    P4=new Point3d(xMin,valPlan,zMax);
	    planCoupe=new Face(P1,P2,P3,P4);
	}
	else if (modePlan==3) {
	    P1=new Point3d(xMin,yMin,valPlan);
	    P2=new Point3d(xMax,yMin,valPlan);
	    P3=new Point3d(xMax,yMax,valPlan);
	    P4=new Point3d(xMin,yMax,valPlan);
	    planCoupe=new Face(P1,P2,P3,P4);
	}
    }

    public void definirCube(String S) {
	Double[] tab = new Double[6];
	boolean ok=true;
	int n=0;
	StringTokenizer t=new StringTokenizer(S,",",false);
	while (ok && t.hasMoreTokens() && n<6) {
	    tab[n]=new Double(t.nextToken());
	    if (tab[n]==null) ok=false;
	    else n++;
	}
	if (ok && n==6) {
	    xMin=tab[0].doubleValue();
	    xMax=tab[1].doubleValue();
	    yMin=tab[2].doubleValue();
	    yMax=tab[3].doubleValue();
	    zMin=tab[4].doubleValue();
	    zMax=tab[5].doubleValue();
	    repaint();
	}
    }

    public void compile() {
	MathStr MS=new MathStr(defSurface.toUpperCase(),";=");
	if (MS.motCourant.equals("Y")) mode=1;
	else mode=0;
	MS.lireSuivant(); MS.passeEspaces(); //Lire Y ou Z
	MS.lireSuivant(); MS.passeEspaces(); //Lire =
	MathNode.defVar("X",0.0,dico);
	if (mode==0) MathNode.defVar("Y",0.0,dico);
	else MathNode.defVar("Z",0.0,dico);
	formule=MS.evalExp(dico,true);
	if (formule==null) statut=1; else statut=0;
    }

    public void tracePlanCoupe(Graphics g) {
	Point3d P;
	if (planCoupe==null) return;
	g.setColor(couleurDessin);
	Point3d P0=planCoupe.donneSommet(0);
	traceVers(g,P0.x,P0.y,P0.z,0);
	for (int i=1; i<4; i++) {
	    P=planCoupe.donneSommet(i);
	    traceVers(g,P.x,P.y,P.z,1);
	}
	traceVers(g,P0.x,P0.y,P0.z,1);
    }

    public void boite(Graphics g) {
	couleur=couleurCache;
	traceSegment(g, new Point3d(xMin,yMin,zMin), new Point3d(xMax,yMin,zMin));
	traceSegment(g, new Point3d(xMax,yMin,zMin), new Point3d(xMax,yMax,zMin));
	traceSegment(g, new Point3d(xMax,yMax,zMin), new Point3d(xMin,yMax,zMin));
	traceSegment(g, new Point3d(xMin,yMax,zMin), new Point3d(xMin,yMin,zMin));
	traceSegment(g, new Point3d(xMin,yMin,zMax), new Point3d(xMax,yMin,zMax));
	traceSegment(g, new Point3d(xMax,yMin,zMax), new Point3d(xMax,yMax,zMax));
	traceSegment(g, new Point3d(xMax,yMax,zMax), new Point3d(xMin,yMax,zMax));
	traceSegment(g, new Point3d(xMin,yMax,zMax), new Point3d(xMin,yMin,zMax));
	traceSegment(g, new Point3d(xMin,yMin,zMin), new Point3d(xMin,yMin,zMax));
	traceSegment(g, new Point3d(xMax,yMin,zMin), new Point3d(xMax,yMin,zMax));
	traceSegment(g, new Point3d(xMax,yMax,zMin), new Point3d(xMax,yMax,zMax));
	traceSegment(g, new Point3d(xMin,yMax,zMin), new Point3d(xMin,yMax,zMax));
	traceSegment(g, new Point3d(xMin,0,0), new Point3d(xMax,0,0));
	traceSegment(g, new Point3d(0,yMin,0), new Point3d(0,yMax,0));
	traceSegment(g, new Point3d(0,0,zMin), new Point3d(0,0,zMax));
    }

    public double calcZ(double x, double y) {
	erreurCalc=false;
	MathNode.defVar("X",x,dico);
	MathNode.defVar("Y",y,dico);
	double z=formule.donneValeur();
	erreurCalc=Div.errCalc(z);
	if (erreurCalc) return 0.0; else return z;
    }

    public double calcY(double x, double z) {
	erreurCalc=false;
	MathNode.defVar("X",x,dico);
	MathNode.defVar("Z",z,dico);
	double y=formule.donneValeur();
	erreurCalc=Div.errCalc(z);
	if (erreurCalc) return 0.0; else return y;
    }

    public void defCouleur(Graphics g, double z, boolean b) {
	if (b) {
	    double val=(zMax-z)*255.0/(zMax-zMin);
	    int ival=(int)val;
	    g.setColor(new Color(128,ival,0));
	}
	else g.setColor(couleurDessin);
    }

    public void ligneX(Graphics g, double x, boolean b) {
	double y, z;
	double y0=0;
	double z0=0;
	double y1=0;
	double z1=0;
	double k;
	int drapeau=0; //nombre de points valides
	int dehors=1;  //hors cube
	for (int j=0; j<=nbPoints; j++) {
	    y=yMin+j*(yMax-yMin)/nbPoints;
	    z=calcZ(x,y);
	    if (!erreurCalc) {
		drapeau++;
		if (drapeau==1) {
		    y0=y; z0=z;
		    if (z>zMax || z<zMin) dehors=1;
		    else { // on est dans le cube
			dehors=0;
			traceVers(g,x,y,z,0);
		    }
		}
		else {
		    drapeau=2;
		    if (dehors==1) { // on est en dehors
			if (z>zMax || z<zMin) { // on reste en dehors
			    y0=y; z0=z;
			}
			else { // on entre dans le cube
			    if (z0>zMax) {
				k=(zMax-z)/(z0-z);
				z0=zMax;
			    }
			    else {
				k=(zMin-z)/(z0-z);
				z0=zMin;
			    }
			    y0=y+k*(y0-y);
			    traceVers(g,x,y0,z0,0);
			    defCouleur(g,(z+z0)/2,b);
			    if (planCoupe!=null && planCoupe.cache(new Point3d(x,y,z),R))
				traceVers(g,x,y,z,0);
			    else traceVers(g,x,y,z,1);
			    y0=y; z0=z;
			    dehors=0;
			}
		    } //fin dehors==1
		    else { //on est dans le cube
			y1=y; z1=z;
			if (z>zMax || z<zMin) { // on sort du cube
			    dehors=1;
			    if (z>zMax) {
				k=(zMax-z)/(z0-z);
				z=zMax;
			    }
			    else {
				k=(zMin-z)/(z0-z);
				z=zMin;
			    }
			    y=y+k*(y0-y);
			}
			defCouleur(g,(z+z0)/2,b);
			if (planCoupe!=null && planCoupe.cache(new Point3d(x,y,z),R))
			    traceVers(g,x,y,z,0);
			else traceVers(g,x,y,z,1);
			y0=y1; z0=z1;
		    }
		}
	    } // fin !erreurCalc
	    else drapeau=0;
	} // for j
    }

    public void ligneY(Graphics g, double y, boolean b) {
	double x, z;
	double x0=0;
	double z0=0;
	double x1=0;
	double z1=0;
	double k;
	int drapeau=0; //nombre de points valides
	int dehors=1;  //hors cube
	for (int j=0; j<=nbPoints; j++) {
	    x=xMin+j*(xMax-xMin)/nbPoints;
	    z=calcZ(x,y);
	    if (!erreurCalc) {
		drapeau++;
		if (drapeau==1) {
		    x0=x; z0=z;
		    if (z>zMax || z<zMin) dehors=1;
		    else { // on est dans le cube
			dehors=0;
			traceVers(g,x,y,z,0);
		    }
		}
		else { // il y a dj un point
		    drapeau=2;
		    if (dehors==1) { // on est en dehors
			if (z>zMax || z<zMin) { // on reste en dehors
			    x0=x; z0=z;
			}
			else { // on entre dans le cube
			    if (z0>zMax) {
				k=(zMax-z)/(z0-z);
				z0=zMax;
			    }
			    else {
				k=(zMin-z)/(z0-z);
				z0=zMin;
			    }
			    x0=x+k*(x0-x);
			    traceVers(g,x0,y,z0,0);
			    defCouleur(g,(z+z0)/2,b);
			    if (planCoupe!=null && planCoupe.cache(new Point3d(x,y,z),R))
				traceVers(g,x,y,z,0);
			    else traceVers(g,x,y,z,1);
			    x0=x; z0=z;
			    dehors=0;
			}
		    } //fin dehors==1
		    else { //on est dans le cube
			x1=x; z1=z;
			if (z>zMax || z<zMin) { // on sort du cube
			    dehors=1;
			    if (z>zMax) {
				k=(zMax-z)/(z0-z);
				z=zMax;
			    }
			    else {
				k=(zMin-z)/(z0-z);
				z=zMin;
			    }
			    x=x+k*(x0-x);
			}
			defCouleur(g,(z+z0)/2,b);
			if (planCoupe!=null && planCoupe.cache(new Point3d(x,y,z),R))
			    traceVers(g,x,y,z,0);
			else traceVers(g,x,y,z,1);
			x0=x1; z0=z1;
		    }
		}
	    } // fin !erreurCalc
	    else drapeau=0;
	} // for j

    }

    public void ligneZ(Graphics g, double z) {
	double x, y;
	double x0=0;
	double y0=0;
	double x1=0;
	double y1=0;
	double k;
	int drapeau=0; //nombre de points valides
	int dehors=1;  //hors cube
	defCouleur(g,z,true);
	for (int j=0; j<=nbPoints; j++) {
	    x=xMin+j*(xMax-xMin)/nbPoints;
	    y=calcY(x,z);
	    if (!erreurCalc) {
		drapeau++;
		if (drapeau==1) {
		    x0=x; y0=y;
		    if (y>yMax || y<yMin) dehors=1;
		    else { // on est dans le cube
			dehors=0;
			traceVers(g,x,y,z,0);
		    }
		}
		else { // il y a dj un point
		    if (dehors==1) { // on est en dehors
			if (y>yMax || y<yMin) { // on reste en dehors
			    x0=x; y0=y;
			}
			else { // on entre dans le cube
			    if (y0>yMax) {
				k=(yMax-y)/(y0-y);
				y0=yMax;
			    }
			    else {
				k=(yMin-y)/(y0-y);
				y0=yMin;
			    }
			    x0=x+k*(x0-x);
			    traceVers(g,x0,y0,z,0);
			    if (planCoupe!=null && planCoupe.cache(new Point3d(x,y,z),R))
				traceVers(g,x,y,z,0);
			    else traceVers(g,x,y,z,1);
			    x0=x; y0=y;
			    dehors=0;
			}
		    } //fin dehors==1
		    else { //on est dans le cube
			y1=y; x1=x;
			if (y>yMax || y<yMin) { // on sort du cube
			    dehors=1;
			    if (y>yMax) {
				k=(yMax-y)/(y0-y);
				y=yMax;
			    }
			    else {
				k=(yMin-y)/(y0-y);
				y=yMin;
			    }
			    x=x+k*(x0-x);
			}
			if (planCoupe!=null && planCoupe.cache(new Point3d(x,y,z),R))
			    traceVers(g,x,y,z,0);
			else traceVers(g,x,y,z,1);
			x0=x1; y0=y1;
		    }
		}
	    } // fin !erreurCalc
	    else drapeau=0;
	} // for j

    }

    public void traceSurface(Graphics g) {
	g.setColor(couleurDessin);
	if (planCoupe!=null) tracePlanCoupe(g);
	if (mode==0) {
	    for (int i=0; i<=nbLignes; i++) ligneX(g, xMin+i*(xMax-xMin)/nbLignes, true);
	    for (int i=0; i<=nbLignes; i++) ligneY(g, yMin+i*(yMax-yMin)/nbLignes, true);
	    if (modePlan==1) ligneX(g,valPlan,false);
	    if (modePlan==2) ligneY(g,valPlan,false);
	}
	else {
	    for (int i=0; i<=2*nbLignes; i++) ligneZ(g, zMin+i*(zMax-zMin)/(2*nbLignes));
	}
    }

    public void dessineFig(Graphics g) {
	g.setColor(couleurFond);
	g.fillRect(0,0,R.largeur,R.hauteur);
	boite(g);
	if (formule!=null) traceSurface(g);
    }

}
