/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 *
 * Courbe.java
 * JDK 1.1
 * Version 1.1 Rvision 03/06/2000
 * Partie du package calkul reprsentant une courbe.
 *
 */

package jkb.geom;

import java.util.Hashtable;
import java.awt.*;
import jkb.utils.Div;
import jkb.utils.Couleurs;
import jkb.calkul.*;

/**
 * Classe gnrale reprsentant une courbe.
 */
public class Courbe {

 /**
  * Constantes pour types de courbe
  */
 public final static int cCart=0;    // y=f(x)
 public final static int cPol=1;     // courbe polaire
 public final static int cParam=2;   // courbe paramtrique
 public final static int cErreur=3;  // courbe mal dfinie

 /**
  * Type de la courbe
  */
 public int cType;

 /**
  * Couleur du trac
  */
 public Color couleurTrace;

 /**
  * Chaine de dfinition
  */
 public String DefStr="";

 /**
  * Indicateur d'erreur de calcul
  */
 boolean erreurCalc=false;

 /**
  * Dfinition de la courbe  tracer
  */
 public MathNode f1=null;
 public MathNode f2=null;

 /**
  * Limites et pas pour le paramtre
  */
 public double tMini=0.0;
 public double tMaxi=-1.0;
 public int nombrePoints=500;
 public boolean autoLimite=true;

 /**
  * Point courant
  */
 double xu=0.0;
 double yu=0.0;

 /**
  * Constructeur
  */
 public Courbe(String Def, Hashtable dico) {
  String tmp;
  MathStr MS;
  cType=cErreur;
  DefStr=Def;
  MS=new MathStr(Def.toUpperCase(),";=");
  switch (MS.suivant) {
   case 'Y' :
    MS.lireSuivant(); MS.passeEspaces(); //Lire Y
    MS.lireSuivant(); MS.passeEspaces(); //Lire =
    MathNode.defVar("X",0.0,dico);
    f1=MS.evalExp(dico,true);
    if (f1!=null) cType=cCart;
    break;
   case 'R' :
    MS.lireSuivant(); MS.passeEspaces(); //Lire R
    MS.lireSuivant(); MS.passeEspaces(); //Lire =
    MathNode.defVar("T",0.0,dico);
    f1=MS.evalExp(dico,true);
    if (f1!=null) cType=cPol;
    break;
   case 'X' :
    MS.lireSuivant(); MS.passeEspaces(); //Lire X
    MS.lireSuivant(); MS.passeEspaces(); //Lire =
    MathNode.defVar("T",0.0,dico);
    f1=MS.evalExp(dico,false);
    if (f1!=null) {
     MS.lireSuivant(); MS.passeEspaces(); //Lire ;
     MS.lireSuivant(); MS.passeEspaces(); //Lire Y
     MS.lireSuivant(); MS.passeEspaces(); //Lire =
     f2=MS.evalExp(dico,true);
     if (f2!=null) {
      cType=cParam;
      tMini=-10.0; tMaxi=10.0;
     }
    }
    break;
   } 
  erreurCalc=false;
  couleurTrace = Color.black;
 }
 
 public void fixeCouleur(String nom) {
  couleurTrace=Couleurs.donneCouleur(nom.toUpperCase());
 }

 public void calcXY(double t, Hashtable dico) {
  erreurCalc=false;
  if (cType==cCart) {
   MathNode.defVar("X",t,dico);
   xu=t;
   yu=f1.donneValeur();
   erreurCalc=Div.errCalc(yu);
  }
  else if (cType==cPol) {
   MathNode.defVar("T",t,dico);
   double d=f1.donneValeur();
   erreurCalc=Div.errCalc(d);
   if (!erreurCalc) {
    xu=d*Math.cos(t);
    yu=d*Math.sin(t);
   } 
  }
  else if (cType==cParam) {
   MathNode.defVar("T",t,dico);
   xu=f1.donneValeur();
   erreurCalc=Div.errCalc(xu);
   if (!erreurCalc) {
    yu=f2.donneValeur();
    erreurCalc=Div.errCalc(yu);
   }
  } 
 }
 
 public void defLimites(Plan p, Hashtable dico) {
  double r,r1;
  boolean ok=true;
  if (cType==cCart) {
   tMini=p.xMin();
   tMaxi=p.xMax();
  }
  else if (cType==cPol) {
   tMaxi=0;
   double t=0.5;
   MathNode.defVar("T",t,dico);
   r1=f1.donneValeur();
   if (!Div.errCalc(r1)) {
    do {
     tMaxi=tMaxi+2*Math.PI;
     t=t+2*Math.PI;
     MathNode.defVar("T",t,dico);
     r=f1.donneValeur();
     ok= !Div.errCalc(r);
    } while ((Math.abs(r1-r)>1E-6) && (tMaxi<100*Math.PI) & (ok));
   } 
   if (!ok) {
    tMini=-50*Math.PI;
    tMaxi=50*Math.PI;
   } 
   else tMini=0.0;
  }
 }

 public void trace(Graphics g, jkb.geom.Plan p, Hashtable dico) {
  // procdure principale : effectue le trac de la courbe
  int PlotFlag;  // drapeau indiquant comment relier 2 points 
                 // PlotFlag=0 pour le premier point  trouver 
                 // PlotFlag=1 si le dernier point tait dans la fentre 
                 // PlotFlag=2 si le dernier point tait  l'extrieur
  double t;      // paramtre
  double pas;    // variation du paramtre
  if (cType==cErreur) {
   g.drawString("Courbe mal dfinie.",10,10);
   return;
  } 
  int ma=MathNode.modeAngle;
  MathNode.modeAngle=MathNode.mRadian;
  PlotFlag=0;
  g.setColor(couleurTrace);
  if (autoLimite) defLimites(p,dico);
  t=tMini;
  pas=(tMaxi-tMini)/nombrePoints;
  // boucle de trac 
  do {
   calcXY(t,dico);
   if (!erreurCalc) {
    // si le calcul s'est effectu correctement
    switch (PlotFlag) {
     case 1: 
      p.traceVers(g,xu,yu,1); // on relie au dernier point
      if (p.dedans(xu,yu)==false) PlotFlag=2; // on indique la sortie
      break;
     case 2: 
      if (p.dedans(xu,yu)) {
       // on revient dans la fentre
       PlotFlag=1;  // graphique cran 
       p.traceVers(g,xu,yu,1);  // relier
      } 
      else p.traceVers(g,xu,yu,0);
      break;
     case 0: 
      p.traceVers(g,xu,yu,0);
      if (p.dedans(xu,yu)) PlotFlag=1;  // tracer le premier point 
      else PlotFlag=2;                // premier point  l'extrieur
      break;
    }  
   }
   else {
    PlotFlag=0;     // on n'est pas dans le domaine de dfinition
   } 
  t=t+pas;             // valeur suivante du paramtre xu 
  } while (t<tMaxi+pas);   // condition d'arrt du trac
  MathNode.modeAngle=ma;
 }
 
 
}

