/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * Plan.java
 * JDK 1.1  
 * Version 1.1 Rvision 28/07/2000
 * Partie du package jkb.geom fournissant des fonctions de calculs
 * gomtrique.
 *
 */

package jkb.geom;

public class GeoCalc {

 /**
  * Valeur utilise pour tester des galits.
  */
 public static double zero = 0.00000001;

 private GeoCalc() {
 }
 
 /**
  * Transforme un nombre de type double (r) en Integer
  */
 public final static int recadre(double r) {
  if (Math.abs(r)<Integer.MAX_VALUE) return (int)Math.round(r);
  else if (r>0) return Integer.MAX_VALUE;
  else return Integer.MIN_VALUE;
 }

 /**
  * Test de nullit utilisant la constante Zero.
  */
 public final static boolean estNul(double r) {
  if (Math.abs(r)<zero) return true;
  else return false;
 } 

 /**
  * Distance entre les points de coordonnes (x1,y1) et (x2,y2).
  */
 public final static double dist(double x1, double y1, double x2, double y2) {
  return Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
 } 

 /**
  * Angle gomtrique entre les vecteurs (x1,y1) et (x2,y2).
  */
 public final static double theta(double x1, double y1, double x2, double y2) {
  double r;
  if (x1==x2) {
   if (y2>=y1) r=Math.PI/2;
   else r=-Math.PI/2;
  } 
  else {
   r=Math.atan((y1-y2)/(x1-x2));
   if (x2>=x1) {
    if (r<0) r=2*Math.PI+r;
   } 
   else r=Math.PI+r;
  }
  return r;
 }
 
}
