/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * Plan.java
 * JDK 1.1  
 * Version 1.1 Rvision 28/07/2000
 * Partie du package jkb.geom fournissant un objet reprsentant
 * un plan muni d'un repre.
 *
 */

package jkb.geom;

import java.awt.*;

/**
 * Classe reprsentant un plan muni d'un repre.
 */
public class Plan extends Canvas {


// fentre courante pour l'utilisateur
 private double xgFen;
 private double xdFen;
 private double ybFen;
 private double yhFen;
 // fonte pour crire
 private final static String FONTE_TEXTE = "SansSerif";
 // point courant pour traceVers
 private int xcur = 0;
 private int ycur = 0;
 //dimensions courantes
 private int oldWidth = 0;
 private int oldHeight = 0;
 // image contenant la figure utilise pour le double buffering
 public Image img;

 /**
  * Nombre par dfaut de pixels par unit pour le repre de l'objet
  * Plan.
  */
 public static double defUnX = 20.0;

 /**
  * Dfinir les couleurs : fond, axes, quadrillage.
  */
 public Color couleurFond;
 public Color couleurAxes;
 public Color couleurQuadrillage;

 /**
  * Options axes et quadrillage
  */
 public boolean optAxes;
 public boolean optQuadrillage;

 /**
  * Nombre de pixels par unit sur les axes.
  */
 protected double unX = defUnX;
 protected double unY = defUnX;

 /**
  * Pas de graduation des axes
  */
 public double xGrad;
 public double yGrad;

 /**
  * Constructeur pour une zone par dfaut de 200x200
  */
 public Plan() {
  setSize(200,200);
  couleurFond = Color.white;
  couleurAxes = Color.black;
  couleurQuadrillage = Color.lightGray;
  optQuadrillage = false;
  optAxes = false;
  xGrad=1.0;
  yGrad=1.0;
  fenetreOrthoCentre(defUnX);
  setFont(new Font(FONTE_TEXTE,Font.BOLD,12));
  img=null;
 }

 /**
  * Supprime le dessin automatique du fond.
  */
 public void update(Graphics g) {
  paint(g);
 }
 public boolean isDoubleBuffered() {
  return true;
 }
 public boolean isOpaque() {
  return true;
 }

 /**
  * Renvoie un contexte graphique pour le plan
  */
 public Graphics donneCG() {
  if (img==null || oldWidth!=getSize().width || oldHeight!=getSize().height) {
   oldWidth=getSize().width;
   oldHeight=getSize().height;
   img=createImage(oldWidth, oldHeight);
  }
  return img.getGraphics();
 }

 /**
  * Procdure d'affichage.
  */
 public void paint(Graphics g) {
  if (img!=null) g.drawImage(img,0,0,this);
  else dessineFond(g);
 }

 /**
  * Dessin du fond.
  */
 public void dessineFond(Graphics g) {
  g.setColor(couleurFond);
  g.fillRect(0,0,getSize().width,getSize().height);
  if (optQuadrillage) quadrille(g);
  if (optAxes) axes(g);
 }

 /**
  * Dessin des axes
  */
 void axes(Graphics g) {
  int dep=2;
  int xs1,ys1,xs2,ys2;
  double t;
  g.setColor(couleurAxes);
  traceVers(g,xMin(),0.0,0);
  traceVers(g,xMax(),0.0,1);
  traceVers(g,0.0,yMin(),0);
  traceVers(g,0.0,yMax(),1);
  //graduation de l'axe des abscisses
  ys1=yEcran(0)-dep;
  ys2=ys1+2*dep;
  t=xGrad;
  while (t<xMax()) {
   xs1=xEcran(t);
   g.drawLine(xs1,ys1,xs1,ys2);
   t=t+xGrad;
  }
  t=-xGrad;
  while (t>xMin()) {
   xs1=xEcran(t);
   g.drawLine(xs1,ys1,xs1,ys2);
   t=t-xGrad;
  }
  //graduation de l'axe des ordonnes
  xs1=xEcran(0)-dep;
  xs2=xs1+2*dep;
  t=yGrad;
  while (t<yMax()) {
   ys1=yEcran(t);
   g.drawLine(xs1,ys1,xs2,ys1);
   t=t+yGrad;
  }
  t=-yGrad;
  while (t>yMin()) {
   ys1=yEcran(t);
   g.drawLine(xs1,ys1,xs2,ys1);
   t=t-yGrad;
  }
 }

 /**
  * Dessin du quadrillage
  */
 void quadrille(Graphics g) {
  int dep=2;
  int xs1,ys1,xs2,ys2;
  double t;
  g.setColor(couleurQuadrillage);
  //graduation de l'axe des abscisses
  ys1=0;
  ys2=getHeight();
  t=xGrad;
  while (t<xMax()) {
   xs1=xEcran(t);
   g.drawLine(xs1,ys1,xs1,ys2);
   t=t+xGrad;
  }
  t=-xGrad;
  while (t>xMin()) {
   xs1=xEcran(t);
   g.drawLine(xs1,ys1,xs1,ys2);
   t=t-xGrad;
  }
  //graduation de l'axe des ordonnes
  xs1=0;
  xs2=getWidth();
  t=yGrad;
  while (t<yMax()) {
   ys1=yEcran(t);
   g.drawLine(xs1,ys1,xs2,ys1);
   t=t+yGrad;
  }
  t=-yGrad;
  while (t>yMin()) {
   ys1=yEcran(t);
   g.drawLine(xs1,ys1,xs2,ys1);
   t=t-yGrad;
  }
 }

 /**
  * Renvoie l'abscisse minimale visible.
  */
 public final double xMin() {
  return xgFen;
 }

 /**
  * Renvoie l'abscisse maximale visible.
  */
 public final double xMax() {
  return xdFen;
 }

 /**
  * Renvoie l'ordonne minimale visible.
  */
 public final double yMin() {
  return ybFen;
 }

 /**
  * Renvoie l'ordonne maximale visible.
  */
 public final double yMax() {
  return yhFen;
 }

 /**
  * Convertit l'abscisse cran en abscisse dans le repre.
  */
 public final double xMath(int xs) {
  return xs/unX+xgFen;
 }

 /**
  * Convertit l'ordonne cran en ordonne dans le repre.
  */
 public final double yMath(int ys) {
  return yhFen-ys/unY;
 }

 /**
  * Convertit l'abscisse dans le repre en abscisse cran.
  */
 public final int xEcran(double xu) {
  double t=(xu-xgFen)*unX;
  return GeoCalc.recadre(t);
 }

 /**
  * Convertit l'ordonne dans le repre en ordonne cran.
  */
 public final int yEcran(double yu) {
  double t=(yhFen-yu)*unY;
  return GeoCalc.recadre(t);
 }

 /**
  * Convertit 1 pixel en distance dans le repre sur l'axe
  * des abscisses.
  */
 public final double unPixelX() {
  return 1/unX;
 }

 /**
  * Convertit 1 pixel en distance dans le repre sur l'axe
  * des ordonnes.
  */
 public final double unPixelY() {
  return 1/unY;
 }

 /**
  * Cre un repre avec les abscisses allant de x1  x2 et
  * les ordonnes allant de y1  y2.
  */
 public void fenetre(double x1, double x2, double y1, double y2) {
  xgFen=x1;
  xdFen=x2;
  ybFen=y1;
  yhFen=y2;
  unX=getSize().width/(xdFen-xgFen);
  unY=getSize().height/(yhFen-ybFen);
 }

 /**
  * Cre un repre orthonormal dont l'origine est au centre
  * de la zone cran et dont l'unit est de (u) pixels.
  */
 public void fenetreOrthoCentre(double u) {
  double xx;
  double yy;
  xx=getSize().width/(2*u);
  yy=getSize().height/(2*u);
  fenetre(-xx,xx,-yy,yy);
 }

 /**
  * Cre un repre orthonormal pour lequel xmin=x0 et ymin=y0
  * et dont l'unit est de (u) pixels.
  */
 public void fenetreOrtho(double u, double x0, double y0) {
  double x1;
  double y1;
  x1 = x0 + getSize().width/u;
  y1 = y0 + getSize().height/u;
  fenetre(x0,x1,y0,y1);
 }

/**
 * Indique si un point est dans la partie visible.
 */
 public boolean dedans(double x, double y) {
  return (x>=xMin() && x<=xMax() && y<=yMax() && y>=yMin());
 }

 /**
  * Affichage d'un point avec son nom  la position (x,y) 
  * du repre.
  */
 public void affichePoint(Graphics g, double x, double y, String nom, Color col, int tf) {
  int xt = xEcran(x);
  int yt = yEcran(y);
  g.setColor(col);
  g.drawLine(xt-2,yt,xt+2,yt);
  g.drawLine(xt,yt-2,xt,yt+2);
  if (nom!=null) {
   Font fo=new Font(FONTE_TEXTE,Font.BOLD,tf);
   g.setFont(fo);
   FontMetrics fm=g.getFontMetrics(fo);
   int l=fm.stringWidth(nom);
   int h=fm.getHeight();
   if (x>=0) xt=xt+4; else xt=xt-l-4;
   if (y<0) yt=yt+h-4;
   g.drawString(nom,xt,yt);
  }
 }

 /**
  * Affichage d'un texte  la position (x,y) du repre.
  */
 public void afficheTexte(Graphics g, double x, double y, String txt, Color col, int tf) {
  int xt = xEcran(x);
  int yt = yEcran(y);
  g.setColor(col);
  g.setFont(new Font(FONTE_TEXTE,Font.BOLD,tf));
  g.drawString(txt,xt,yt);
 }

 /**
  * Dessin d'une droite d'quation ax+by+c=0.
  */
 public void afficheDroite(Graphics g, double a, double b, double c, Color col) {
  int x[]=new int[2];
  int y[]=new int[2];
  int np=0;
  double xx,yy;
  if (GeoCalc.estNul(b)) { 
   //droite verticale
   x[0]=xEcran(-c/a);
   y[0]=0;
   x[1]=x[0];
   y[1]=getSize().height;
   np=2;
   }
  else if (GeoCalc.estNul(a)) {
   //droite horizontale
   y[0]=yEcran(-c/b);
   x[0]=0;
   y[1]=y[0];
   x[1]=getSize().width;
   np=2;
   }
  else {
   //intersection avec x=xgFen
   xx=xgFen;
   yy=(-c-a*xx)/b;
   if ((yy>=ybFen) && (yy<=yhFen)) {
    x[np]=xEcran(xx);
    y[np]=yEcran(yy);
    np++;
   }
   //intersection avec x=xdFen
   xx=xdFen;
   yy=(-c-a*xx)/b;
   if ((yy>=ybFen) && (yy<=yhFen)) {
    x[np]=xEcran(xx);
    y[np]=yEcran(yy);
    np++;
   }
   if (np<2) {
    yy=ybFen;
    xx=(-c-b*yy)/a;
    if ((xx>=xgFen) && (xx<=xdFen)) {
     x[np]=xEcran(xx);
     y[np]=yEcran(yy);
     np++;
    }
    if (np<2) {
     yy=yhFen;
     xx=(-c-b*yy)/a;
     if ((xx>=xgFen) && (xx<=xdFen)) {
      x[np]=xEcran(xx);
      y[np]=yEcran(yy);
      np++;
     } 
    }
   }
  } 
  if (np==2) {
   g.setColor(col);
   g.drawLine(x[0],y[0],x[1],y[1]);
  } 
 }
 
 /**
  * Dessin d'un cercle de centre (x,y) et de rayon r.
  */
 public void afficheCercle(Graphics g, double x, double y, double r, Color col) {
  int xs = xEcran(x);
  int ys = yEcran(y);
  int rs = GeoCalc.recadre(r*unX);
  g.setColor(col);
  g.drawOval(xs-rs,ys-rs,2*rs,2*rs);
 }

 /**
  * Dessin d'un segment allant de (xr1,yr1)  (xr2,yr2).
  */
 public void afficheSegment(Graphics g, double xr1, double yr1, double xr2, double yr2, Color col) {
  int x1 = xEcran(xr1);
  int y1 = yEcran(yr1);
  int x2 = xEcran(xr2);
  int y2 = yEcran(yr2);
  g.setColor(col);
  g.drawLine(x1,y1,x2,y2);
 }

 /**
  * Dplacement du point courant : si mode=1 un segment est
  * dessin.
  */
 public void traceVers(Graphics g, double x, double y, int mode) {
  int xe=xEcran(x);
  int ye=yEcran(y);
  if (mode==1) g.drawLine(xcur,ycur,xe,ye);
  xcur=xe;
  ycur=ye;
 }

}

