/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * Droite.java
 * JDK 1.1  
 * Version 1.2 Rvision 01/08/2000
 * Partie du package imageo fournissant l'objet droite qui
 * peut tre dfinie par deux points, une parallle ou une
 * perpendiculaire.
 *
 */

package jkb.imageo;

import java.util.Vector;

/**
 * Classe drive de Segment reprsentant une droite.
 * Le premier paramtre est un point de la droite.
 * La droite peut tre dfinie :
 * - par deux points
 * - comme perpendiculaire passant par un point
 * - comme parallle passant par un point
 **/
public class Droite extends Segment {
 
 /**
  * Initialise une droite dfinie par deux points ou une parallle 
  * ou une perpendiculaire.
  **/
 public Droite(Vector V, int typ) {
  super(typ);
  if ("PP".equals(V.elementAt(V.size()-1)) && typ==iDroite) {
   objs[1]=(ObjImageo)V.elementAt(0);
   objs[2]=(ObjImageo)V.elementAt(1);
   stInitOk=true;
  } 
  else if ("DP".equals(V.elementAt(V.size()-1)) && (typ==iParallele || typ==iPerpendiculaire)) {
   objs[2]=(ObjImageo)V.elementAt(0);
   objs[1]=(ObjImageo)V.elementAt(1);
   stInitOk=true;
  }
  else stInitOk=false;
 }

 /**
  * Calcule les coefficients a, b et c de l'quation de la
  * droite.
  **/
 public void calcDatas() {
  switch (oType) {
   case iParallele :
    if (stInitOk && objs[1].estValide() && objs[2].estValide()) { 
     datas[0]=objs[2].donneA();
     datas[1]=objs[2].donneB();
     datas[2]=-objs[2].donneA()*objs[1].donneX()-objs[2].donneB()*objs[1].donneY();
     stCalcOk=true;
    }
    else stCalcOk=false;
    break;
   case iPerpendiculaire :
    if (stInitOk && objs[1].estValide() && objs[2].estValide()) { 
     datas[0]=objs[2].donneB();
     datas[1]=-objs[2].donneA();
     datas[2]=-objs[2].donneB()*objs[1].donneX()+objs[2].donneA()*objs[1].donneY();
     stCalcOk=true;
    }
    else stCalcOk=false;
    break;
   case iDroite :
    super.calcDatas(); //hritage de Segment
  }
 }
 
} 

