/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyrigand distribute this software and its
 * documentation for NON-COMMERCIAL pht notice appears in all copies.
 * 
 * FigureImageo.java
 * JDK 1.1  
 * Version 1.2a Rvision 28/12/1999
 * Partie du package imageo fournissant une figure Imageo.
 * Version 1.3a Rvision 18/07/2006
 * 
 */

package jkb.imageo;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.Vector;
import jkb.geom.Plan;
import jkb.geom.GeoCalc;
import jkb.utils.Div;

/**
 * Classe reprsentant une figure Imageo comme un vecteur
 * d'objet imageo.
 * Par exemple:
 * <pre>
 *    FigureImageo Fig = new FigureImageo();
 * </pre>
 */
public class FigureImageo extends Vector {

 public final static String ImageoVersion = "Imageo v1.3b (c) 1999-2000 Bruno K.";

 /**
  * Contient les traces de points.
  */
 Trace trace;
 
 /**
  * Couleur d'affichage par dfaut des objets gomtriques.
  */
 Color couleurDessin = Color.white;
 /**
  * Couleur d'affichage des objets slectionns.
  */
 Color couleurChoisi = Color.red;
 /**
  * Couleur d'affichage des points mobiles.
  */
 Color couleurMobile = Color.yellow;
 /**
  * Couleur d'affichage des mesures.
  */
 Color couleurTexte = Color.white;
 /**
  * Couleur d'affichage des traces
  */
 Color couleurTrace = new Color(0x00FFFF);
 /**
  * Couleur d'affichage du fond
  */
 Color couleurFond = new Color(0x008000);

 /** 
  * Taille des caractres pour le texte
  */
 int tailleFonte = 12;

 /**
  * Constructeur.
  */
 public FigureImageo() {
  super(100,50);  
  trace=null;
 }

 /**
  * Suppression de tous les objets de la figure et
  * rtablissement des couleurs d'origine.
  */
 public final void vider() {
  removeAllElements();
  if (trace!=null) trace.removeAllElements();
  couleurDessin = Color.white;
  couleurChoisi = Color.red;
  couleurMobile = Color.yellow;
  couleurTexte = Color.white;
  couleurFond = new Color(0x008000);
  couleurTrace = new Color(0x00FFFF);
 }

 /**
  * Renvoie l'objet de numro num.
  */
 public final ObjImageo objNum(int num) {
  ObjImageo o;
  if (num<0 || num>=size()) o=null;
  else o=(ObjImageo)elementAt(num);
  return o;
 }

 /**
  * Recalcule les donnes d'affichage de tous les objets de la
  * figure.
  */
 public final void calcFigure() {
  int i;
  ObjImageo o;
  for (i=0; i<size(); i++) {
   o=objNum(i);
   if (o.stInitOk) o.calcDatas(); 
  }
 }
 
 /**
  * Affichage de la figure.
  */
 public void dessineFigure(Graphics g,Plan P) {
  int k=0;
  ObjImageo o;
  Color col;
  FontMetrics fm;
  String S="";
  double[] m=new double[2];
  //affichage des lignes
  for (k=0; k<size(); k++) {
   o=objNum(k);
   if (o!=null && o.estValide() && o.stVisible && o.oType>=300) {
    if (o.stChoisi) col=couleurChoisi;
    else if (o.stMobile) col=couleurMobile;
    else col=couleurDessin;
    if (o.stPointille) col=(col.brighter()).brighter();
    if (o.stEpais) col=(col.darker()).darker();
    switch (o.donneType()) {
     case ObjImageo.iDroite : 
      P.afficheDroite(g,o.donneA(),o.donneB(),o.donneC(),col);
      break;
     case ObjImageo.iCercle : 
      P.afficheCercle(g,o.donneX(),o.donneY(),o.donneR(),col);
      break;
     case ObjImageo.iSegment :
      P.afficheSegment(g,o.objs[1].donneX(),o.objs[1].donneY(),
       o.objs[2].donneX(),o.objs[2].donneY(),col);
      break;
    }
   } 
  }
  //affichage de la trace
  if (trace!=null) {
   for (k=0; k<trace.size(); k++) {
    m=trace.coord(k);
    g.setColor(couleurTrace);
    g.fillOval(P.xEcran(m[0])-1,P.yEcran(m[1])-1,3,3);
    //P.traceVers(g,m[0],m[1],0);
    //P.traceVers(g,m[0],m[1],1);
   }
  }
  //affichage des points et des mesures
  for (k=0; k<size(); k++) {
   o=objNum(k);
   if (o!=null && o.donneType()==ObjImageo.iPoint) {
    if (o.estValide() && o.stVisible) {
     if (o.stChoisi) col=couleurChoisi;
     else if (o.stMobile) col=couleurMobile;
     else if (o.stTrace) col=couleurTrace;
     else col=couleurDessin;
     P.affichePoint(g,o.donneX(),o.donneY(),o.nom,col,tailleFonte);
     if (o.stTrace) {
      if (trace==null) trace=new Trace();
      trace.ajoute(o.donneX(),o.donneY());
     } 
    } 
   }
   else if (o!=null && o.donneType()==ObjImageo.iMesure) {
    if (o.estValide() && o.stVisible) {
     col=couleurTexte;
     if (o.oType==ObjImageo.iMesureSegment)
      P.afficheTexte(g,o.donneX(),o.donneY(),Div.DoubleToStr(o.donneR(),2),col,tailleFonte);
     else {
      S=Div.DoubleToStr(o.donneR(),1)+"";
      fm=g.getFontMetrics();
      P.afficheTexte(g,o.donneX()-0.5*fm.stringWidth(S)*P.unPixelX(),o.donneY(),S,col,tailleFonte);
     } 
    }    
   }
  }
 }

 
 /**
  * Renvoie l'objet de nom S ou null si aucun objet n'a ce nom.
  */
 public final ObjImageo trouveObj(String S) {
  int i=0;
  ObjImageo o=null;
  boolean ok=false;
  while ((!ok) && (i<size())) {
   o=objNum(i);
   if ((o!=null) && (S.compareTo(o.nom)==0)) ok=true;
   else i++;
  }
  if (ok) return o;
  else return null;
 }
 
 /**
  * Ajoute l'objet o  la figure en lui attribuant son nom;
  * si ce nom est non vide et ne commence pas par _ , l'ajout
  * n'est effectu que si le nom n'est pas dj utilis.
  */
 public final boolean ajouteObj(String nom, ObjImageo o) {
  ObjImageo o1=null;
  if (nom.length()>0 && !nom.startsWith("_")) o1=trouveObj(nom);
  if (o1==null) {
   o.nom=nom;
   addElement(o);
   return true;
  }
  else return false;
 }
 

}

