/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * Mesure.java
 * JDK 1.1  
 * Version 1.2 Rvision 01/08/2000
 * Partie du package imageo permettant de dfinir les objets
 * reprsentant les longueurs de segments et les mesures 
 * d'angles affiches.
 *
 */

package jkb.imageo;

import java.util.Vector;
import jkb.geom.GeoCalc;

/**
 * Classe reprsentant les objets permettant d'afficher les
 * longueurs de segments et les mesures d'angles.
 */
public class Mesure extends ObjImageo {
 
 /**
  * Initialisation de l'affichage de la longueur d'un segment.
  */
 public Mesure(Vector V) {
  super(iMesure);
  if ("PP".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   objs[2]=(ObjImageo)V.elementAt(1);
   stInitOk=true;
   oType=iMesureSegment;
  }
  else if ("PPP".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   objs[2]=(ObjImageo)V.elementAt(1);
   objs[3]=(ObjImageo)V.elementAt(2);
   stInitOk=true;
   oType=iMesureAngle;  
  }
  else stInitOk=false;
 }

 /**
  * Calcul des coordonnes d'affichage et de la valeur affiche.
  */
 public void calcDatas() {
  double a1,a2,r;
  if (oType==iMesureSegment) {
   if (stInitOk && objs[1].estValide() && objs[2].estValide()) { 
    stCalcOk=true; 
    datas[0]=(objs[1].donneX()+objs[2].donneX())/2.0;
    datas[1]=(objs[1].donneY()+objs[2].donneY())/2.0;
    datas[2]=GeoCalc.dist(objs[1].donneX(),objs[1].donneY(),
             objs[2].donneX(),objs[2].donneY());
   }
   else stCalcOk=false;
  }
  else if (oType==iMesureAngle) {
   if (stInitOk && objs[1].estValide() && objs[2].estValide() && objs[3].estValide()) {
    stCalcOk=true;
    a1=GeoCalc.dist(objs[2].donneX(),objs[2].donneY(),objs[1].donneX(),objs[1].donneY());
    a2=GeoCalc.dist(objs[2].donneX(),objs[2].donneY(),objs[3].donneX(),objs[3].donneY());
    if (GeoCalc.estNul(a1) || GeoCalc.estNul(a2)) stCalcOk=false;
    else {
     a1=GeoCalc.theta(objs[2].donneX(),objs[2].donneY(),objs[1].donneX(),objs[1].donneY());
     a2=GeoCalc.theta(objs[2].donneX(),objs[2].donneY(),objs[3].donneX(),objs[3].donneY());
     r=a2-a1;
     while (r<0) r=r+2*Math.PI;
     if (r>Math.PI) {
      r=2*Math.PI-r;
      a1=a1-r/2.0;
     }
     else a1=a1+r/2.0;
     datas[0]=objs[2].donneX()+1.5*Math.cos(a1);
     datas[1]=objs[2].donneY()+1.5*Math.sin(a1);
     datas[2]=r*180/Math.PI;
    } 
   }
   else stCalcOk=false;
  }
  else stCalcOk=false;
 }

} 

