/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * PlanImageo.java
 * JDK 1.1  
 * Version 1.1 Rvision 28/12/1999
 * Partie du package imageo fournissant un composant driv de Plan 
 * pouvant contenir une figure imageo dformable.
 *
 */

package jkb.imageo;

import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import jkb.geom.Plan;
import jkb.geom.GeoCalc;

/**
 * Classe reprsentant un composant contenant une figure
 * Imageo.
 * Par exemple:
 * <pre>
 *    PlanImageo Fig = new PlanImageo();
 * </pre>
 */
public class PlanImageo extends jkb.geom.Plan 
    implements MouseListener, MouseMotionListener {
    
 private jkb.imageo.Point PSel;
 private boolean Mobile;  
 public FigureImageo Fig;
 public boolean optionOrtho=true;
 
 public PlanImageo() {
  super();
  Fig=new FigureImageo();
  PSel=null;
  Mobile=true;
  addMouseListener(this);
  addMouseMotionListener(this);
 }
 
 /**
  * Permet d'indiquer si les points mobiles sont dplaables
  *  la souris.
  */
 public final void rendreMobile(boolean b) {
  Mobile=b;
  if (!Mobile) Fig.couleurMobile=Fig.couleurDessin;
 }

 /**
  * Renvoie le point slectionn  la souris.
  */
 public final jkb.imageo.Point getSel() {
  return PSel;
 }
 
 /**
  * Cherche un point  slectionner quand on a appuy 
  * sur le bouton de la souris.
  */
 public void mousePressed(MouseEvent evt) {
  int i=0;
  ObjImageo o = null;
  PSel=null;
  if (Mobile) {
   boolean ok=false;
   double d=-1.0;
   double xr=xMath(evt.getX());
   double yr=yMath(evt.getY());
   for (i=0; i<Fig.size(); i++) {
    o=Fig.objNum(i);
    if (o!=null && (o.donneType()==ObjImageo.iPoint) && o.estValide() 
       && o.stVisible && o.stMobile) {
     d=GeoCalc.dist(o.donneX(),o.donneY(),xr,yr);
     if (Math.abs(d)<=3*unPixelX()) PSel=(jkb.imageo.Point)o;
    } 
   }
   if (PSel!=null) {
    PSel.stChoisi=true;
    repaint();
   } 
  }
 }

 /**
  * Dplacement du point slectionn en suivant les dplacements
  * de la souris.
  */
 public void mouseDragged(MouseEvent evt) {
  int x=evt.getX();
  int y=evt.getY();
  if (Mobile && PSel!=null && contains(x,y)) {
   PSel.bougeVers(xMath(x),yMath(y));
   repaint();
  }
 }

 /**
  * Dselection du point slectionn s'il y a lieu lorsqu'on
  * relche le bouton de la souris.
  */
 public void mouseReleased(MouseEvent evt) {
  if (Mobile && PSel!=null) {
   PSel.stChoisi=false;
   repaint();
  } 
  PSel=null;
 }

 // vnements souris ignors
 public void mouseEntered(MouseEvent evt) {
 }
 
 public void mouseExited(MouseEvent evt) {
 }
 
 public void mouseClicked(MouseEvent evt) {
 }
 
 public void mouseMoved(MouseEvent evt) {
 }

 /**
  * Procdure d'affichage avec double buffering.
  */
 public void paint(Graphics g) {
  if (optionOrtho) {
   fenetreOrthoCentre(unX);
   optionOrtho=false;
  }
  else fenetre(xMin(),xMax(),yMin(),yMax());
  Graphics FigG=donneCG();
  couleurFond=Fig.couleurFond;
  dessineFond(FigG);
  Fig.calcFigure();
  Fig.dessineFigure(FigG,this);
  super.paint(g);
  FigG.dispose();
 }

}

