/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * PointConstruit.java
 * JDK 1.1  
 * Version 1.2 Rvision 01/08/2000
 * Partie du package imageo fournissant les points obtenus par
 * construction.
 *
 */

package jkb.imageo;

import java.util.Vector;

/**
 * Classe reprsentant les points obtenus par construction, ou
 * comme image d'un autre point par une transformation.
 */
public class PointConstruit extends jkb.imageo.Point {
 
 /**
  * Constructeur; rend le point non mobile.
  */
 public PointConstruit(Vector V, int typ) {
  super(typ);  
  ObjImageo o;
  stMobile=false;
  switch (typ) {
   case iSymetrique :
    if ("PP".equals(V.elementAt(V.size()-1))) {
     objs[1]=(ObjImageo)V.elementAt(0);
     objs[2]=(ObjImageo)V.elementAt(1);
     stInitOk=true;
     oType=iSymPoint; 
    }
    else if ("PD".equals(V.elementAt(V.size()-1))) {
     objs[1]=(ObjImageo)V.elementAt(0);
     objs[2]=(ObjImageo)V.elementAt(1);
     stInitOk=true;
     oType=iSymAxe;
    }
    else stInitOk=false;
    break;
   case iMilieu :
    if ("PP".equals(V.elementAt(V.size()-1))) { 
     objs[1]=(ObjImageo)V.elementAt(0);
     objs[2]=(ObjImageo)V.elementAt(1);
     stInitOk=true;
    }
    else stInitOk=false;
    break;
   case iProjection :
    if ("PD".equals(V.elementAt(V.size()-1))) { 
     objs[1]=(ObjImageo)V.elementAt(0);
     objs[2]=(ObjImageo)V.elementAt(1);
     stInitOk=true;
    }
    else stInitOk=false;
    break;
   case iHomothetie :
   case iRotation :
    if ("PPN".equals(V.elementAt(V.size()-1))) { 
     objs[1]=(ObjImageo)V.elementAt(0);
     objs[2]=(ObjImageo)V.elementAt(1);
     double r=((Double)V.elementAt(2)).doubleValue();
     if (typ==iRotation) datas[2]=r*Math.PI/180.0;
     else datas[2]=r;
     stInitOk=true;
    }
    else stInitOk=false;
    break;
   case iTranslation :
    if ("PPP".equals(V.elementAt(V.size()-1))) {
     objs[1]=(ObjImageo)V.elementAt(0);
     objs[2]=(ObjImageo)V.elementAt(1);
     objs[3]=(ObjImageo)V.elementAt(2);
     stInitOk=true;
    }
    else stInitOk=false;
    break;
  } 
 }

 /**
  * Calcul des coordonnes du point.
  */
 public void calcDatas() {
  double e, aux1, aux2, aux3;
  //3 objets en paramtres
  if (objs[3]!=null) {
   if (stInitOk && objs[1].estValide() && objs[2].estValide() && objs[3].estValide()) {
    stCalcOk=true;
    if (oType==iTranslation) {
     datas[0]=objs[1].donneX()+objs[3].donneX()-objs[2].donneX();
     datas[1]=objs[1].donneY()+objs[3].donneY()-objs[2].donneY();
    }
    else stCalcOk=false;
   }
   else stCalcOk=false;
  }
  //2 objets en paramtres
  else if (objs[2]!=null) {
   if (stInitOk && objs[1].estValide() && objs[2].estValide()) {
    stCalcOk=true;
    switch (oType) {
     case iMilieu :
      datas[0]=(objs[1].donneX()+objs[2].donneX())/2;
      datas[1]=(objs[1].donneY()+objs[2].donneY())/2;
      break;
     case iSymPoint :
      datas[0]=2*objs[2].donneX()-objs[1].donneX();
      datas[1]=2*objs[2].donneY()-objs[1].donneY();
      break;
     case iSymAxe :
      e=objs[2].donneA()*objs[2].donneA()+objs[2].donneB()*objs[2].donneB();
      aux1=objs[2].donneA()*objs[2].donneC();
      aux2=objs[2].donneB()*objs[2].donneA()*objs[1].donneY();
      aux3=objs[2].donneB()*objs[2].donneB()*objs[1].donneX();
      datas[0]=2*(-aux1-aux2+aux3)/e-objs[1].donneX();
      aux1=objs[2].donneA()*objs[2].donneA()*objs[1].donneY();
      aux2=objs[2].donneA()*objs[2].donneB()*objs[1].donneX();
      aux3=objs[2].donneB()*objs[2].donneC();
      datas[1]=2*(aux1-aux2-aux3)/e-objs[1].donneY();
      break;
     case iProjection :
      aux1=objs[2].donneA()*objs[2].donneA()+objs[2].donneB()*objs[2].donneB();
      datas[0]=(-objs[2].donneA()*objs[2].donneC()
               -objs[2].donneB()*objs[2].donneA()*objs[1].donneY()
               +objs[2].donneB()*objs[2].donneB()*objs[1].donneX())/aux1;
      datas[1]=(objs[2].donneA()*objs[2].donneA()*objs[1].donneY()
               -objs[2].donneA()*objs[2].donneB()*objs[1].donneX()
               -objs[2].donneB()*objs[2].donneC())/aux1;
      break;
     case iHomothetie : 
      datas[0]=objs[2].donneX()+datas[2]*(objs[1].donneX()-objs[2].donneX());
      datas[1]=objs[2].donneY()+datas[2]*(objs[1].donneY()-objs[2].donneY());
      break;
     case iRotation :
      aux1=Math.cos(datas[2]);
      aux2=Math.sin(datas[2]);
      datas[0]=objs[2].donneX()+aux1*(objs[1].donneX()-objs[2].donneX())
              -aux2*(objs[1].donneY()-objs[2].donneY());
      datas[1]=objs[2].donneY()+aux2*(objs[1].donneX()-objs[2].donneX())
              +aux1*(objs[1].donneY()-objs[2].donneY());
      break;
     default : stCalcOk=false;
    }
   } 
   else stCalcOk=false;
  } 
  //1 objet en paramtre 
  else if (objs[1]!=null) {
   stCalcOk=false;
  }
 }
 
} 

