/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * PointIntersection.java
 * JDK 1.1  
 * Version 1.2 Rvision 01/08/2000
 * Partie du package imageo permettant de dfinir les points
 * d'intersection de droites et de cercles.
 *
 */

package jkb.imageo;

import java.util.Vector;
import jkb.geom.GeoCalc;

/**
 * Classe reprsentant les points d'intersection de droites
 * et de cercles.
 **/
public class PointIntersection extends jkb.imageo.Point {
 
 /**
  * Constructeur; rend le point non mobile.
  */
 public PointIntersection(Vector V) {
  super(iIntersection);
  stMobile=false;
  if ("DD".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   objs[2]=(ObjImageo)V.elementAt(1);
   stInitOk=true;
   oType=iInterDD;  
  }
  else if ("DCN".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   objs[2]=(ObjImageo)V.elementAt(1);
   datas[2]=((Double)V.elementAt(2)).doubleValue();
   stInitOk=true;
   oType=iInterDC;
  }
  else if ("CCN".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   objs[2]=(ObjImageo)V.elementAt(1);
   datas[2]=((Double)V.elementAt(2)).doubleValue();
   stInitOk=true;
   oType=iInterCC;  
  }
  else if ("DCP".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   objs[2]=(ObjImageo)V.elementAt(1);
   objs[3]=(ObjImageo)V.elementAt(2);
   datas[2]=0;
   stInitOk=true;
   oType=iInterDC;
  }
  else if ("CCP".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   objs[2]=(ObjImageo)V.elementAt(1);
   objs[3]=(ObjImageo)V.elementAt(2);
   datas[2]=0;
   stInitOk=true;
   oType=iInterCC;  
  }
  else stInitOk=false;
 }

 /**
  * Calcul des coordonnes du point.
  */
 public void calcDatas() {
  double a,b,d,e,f,aux;
  double x1,x2,y1,y2;
  if (stInitOk && objs[1].estValide() && objs[2].estValide()) {
   stCalcOk=true;
   switch (oType) {
    case iInterDD :
     d=objs[1].donneA()*objs[2].donneB()-objs[2].donneA()*objs[1].donneB();
     if (GeoCalc.estNul(d)) stCalcOk=false;
     else {
      datas[0]=(objs[1].donneB()*objs[2].donneC()-objs[2].donneB()*objs[1].donneC())/d;
      datas[1]=(objs[2].donneA()*objs[1].donneC()-objs[1].donneA()*objs[2].donneC())/d;
     }
     break;
    case iInterDC :
     aux=GeoCalc.dist(objs[1].donneA(),objs[1].donneB(),0.0,0.0);
     d=(objs[1].donneA()*objs[2].donneX()+objs[1].donneB()*objs[2].donneY()+objs[1].donneC())/aux;
     if (Math.abs(d)>objs[2].donneR()) stCalcOk=false;
     else {
      e=Math.sqrt(objs[2].donneR()*objs[2].donneR()-d*d);
      x1=objs[2].donneX()-(d*objs[1].donneA()-e*objs[1].donneB())/aux;
      y1=objs[2].donneY()-(d*objs[1].donneB()+e*objs[1].donneA())/aux;
      x2=objs[2].donneX()-(d*objs[1].donneA()+e*objs[1].donneB())/aux;
      y2=objs[2].donneY()-(d*objs[1].donneB()-e*objs[1].donneA())/aux;
      if (donneR()==1.0) {
       datas[0]=x1;
       datas[1]=y1;
      }
      else if (donneR()==2.0) {
       datas[0]=x2;
       datas[1]=y2;
      } 
      else {
       aux=GeoCalc.dist(objs[3].donneX(),objs[3].donneY(),x1,y1);
       if (GeoCalc.estNul(aux)) {
       datas[0]=x2;
       datas[1]=y2;
       }
       else {
       datas[0]=x1;
       datas[1]=y1;
       }
      }
     }
     break;
    case iInterCC :
     d=GeoCalc.dist(objs[1].donneX(),objs[1].donneY(),objs[2].donneX(),objs[2].donneY());
     if ( (d>objs[1].donneR()+objs[2].donneR()) || (d<Math.abs(objs[1].donneR()-objs[2].donneR())))
      stCalcOk=false;
     else {
      a=objs[1].donneY()-objs[2].donneY();
      b=objs[2].donneX()-objs[1].donneX();
      aux=GeoCalc.dist(a,b,0.0,0.0);
      e=(objs[1].donneR()*objs[1].donneR()-objs[2].donneR()*objs[2].donneR()+d*d)/(2*d);
      f=Math.sqrt(objs[1].donneR()*objs[1].donneR()-e*e);
      x1=objs[1].donneX()+(b*e+a*f)/aux;
      y1=objs[1].donneY()-(a*e-b*f)/aux;
      x2=objs[1].donneX()+(b*e-a*f)/aux;
      y2=objs[1].donneY()-(a*e+b*f)/aux;
      if (donneR()==1.0) {
       datas[0]=x1;
       datas[1]=y1;
      }
      else if (donneR()==2.0) {
       datas[0]=x2;
       datas[1]=y2;
      }
      else {
       aux=GeoCalc.dist(objs[3].donneX(),objs[3].donneY(),x1,y1);
       if (GeoCalc.estNul(aux)) {
       datas[0]=x2;
       datas[1]=y2;
       }
       else {
       datas[0]=x1;
       datas[1]=y1;
       }
      }
     }
     break;
    default : stCalcOk=false;
   }
  } 
  else stCalcOk=false; 
 }

}

