/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * PointSur.java
 * JDK 1.1
 * Version 1.2 Rvision 01/08/2000
 * Partie du package imageo fournissant l'objet point semi-libre
 * c'est  dire attach  une droite ou  un cercle.
 *
 */

package jkb.imageo;

import java.util.Vector;
import jkb.geom.GeoCalc;

/**
 * Classe reprsentant un point semi-libre, c'est  dire dplaable
 * sur une droite ou sur un cercle.
 */
public class PointSur extends jkb.imageo.Point {
 
 /**
  * Initialisation d'un point li  une droite D1 ou  un cercle
  * avec ou sans paramtre r.
  */
 public PointSur(Vector V) {
  super(iPointSur);
  stMobile=true;
  if ("DN".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   datas[2]=((Double)V.elementAt(1)).doubleValue();
   stInitOk=true;
   oType=iPointSurDroite;
  }
  else if ("D".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   datas[2]=2*Math.random()-1;
   stInitOk=true;
   oType=iPointSurDroite;
  }
  else if ("CN".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   datas[2]=((Double)V.elementAt(1)).doubleValue()*Math.PI/180.0;
   stInitOk=true;
   oType=iPointSurCercle;
  }
  else if ("C".equals(V.elementAt(V.size()-1))) {
   objs[1]=(ObjImageo)V.elementAt(0);
   datas[2]=2*Math.PI*Math.random();
   stInitOk=true;
   oType=iPointSurCercle;
  }
  else stInitOk=false;
 }

 /**
  * Limitation des dplacements pour les points sur segment
  */
 void correctionSegment() {
  if (objs[1].donneType()==iSegment) {
   if (datas[2]<0.0) datas[2]=0.0;
   else if (datas[2]>1) datas[2]=1.0;
  }
 }

 /**
  * Recalcule les coordonnes du point.
  */
 public void calcDatas() {
  if (stInitOk && objs[1].estValide()) {
   stCalcOk=true;
   if (oType==iPointSurDroite) {
    datas[0]=objs[1].objs[1].donneX()-objs[1].donneB()*datas[2];
    datas[1]=objs[1].objs[1].donneY()+objs[1].donneA()*datas[2];
   }
   else if (oType==iPointSurCercle) {
    datas[0]=objs[1].donneX()+objs[1].donneR()*Math.cos(datas[2]);
    datas[1]=objs[1].donneY()+objs[1].donneR()*Math.sin(datas[2]);
   }
  }
  else stCalcOk=false;
 }
 
 /**
  * Dplacement du point.
  */
 public void bougeVers(double x, double y) {
  double xx,yy,e;
  if (stMobile && estValide()) {
   if (oType==iPointSurDroite) {
    e=objs[1].donneA()*objs[1].donneA()+objs[1].donneB()*objs[1].donneB();
    xx=(-objs[1].donneA()*objs[1].donneC()-objs[1].donneB()*objs[1].donneA()*y
        +objs[1].donneB()*objs[1].donneB()*x)/e;
    yy=(objs[1].donneA()*objs[1].donneA()*y-objs[1].donneA()*objs[1].donneB()*x
        -objs[1].donneB()*objs[1].donneC())/e;
    if (!GeoCalc.estNul(objs[1].donneB())) 
     datas[2]=-(xx-objs[1].objs[1].donneX())/objs[1].donneB();
    else datas[2]=(yy-objs[1].objs[1].donneY())/objs[1].donneA();
    correctionSegment();
   }
   else if (oType==iPointSurCercle) {
    if (GeoCalc.estNul(x-objs[1].donneX())) {
     if (y>=objs[1].donneY()) datas[2]=Math.PI/2.0;
     else datas[2]=-Math.PI/2.0;
    } 
    else {
     datas[2]=Math.atan((y-objs[1].donneY())/(x-objs[1].donneX()));
     if (objs[1].donneX()>x) datas[2]=datas[2]+Math.PI;
    } 
   } 
  } 
 }

}

