/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * ComImageo.java
 * JDK 1.1  
 * Version 1.1 Rvision 01/08/2000
 * Partie du package imageo fournissant le dictionnaire des
 * mots cls du langage Imageo et les fonctions de lecture
 * des commandes.
 * La fonction compile() permet d'obtenir une figure imageo.
 *
 */

package jkb.imageo;

import java.util.Vector;
import jkb.utils.ComStr;
import jkb.utils.Couleurs;

/**
 * Classe reprsentant une commande imageo  analyser.
 * Par exemple:
 * <pre>
 *    ScriptImageo scr = new ScriptImageo("A=point B=point segment(A,B)");
 * </pre>
 */
public class ScriptImageo extends ComStr {

 /**
  * codes des commandes imageo
  */
 public static final int oCommande=1000;
 public static final int oCacher=oCommande+1;
 public static final int oFixer=oCommande+2;
 public static final int oChoisir=oCommande+3;
 public static final int oPolygone=oCommande+4;
 public static final int oTrace=oCommande+5;
 public static final int oEpais=oCommande+6;
 public static final int oPointille=oCommande+7;
 public static final int oMixte=oCommande+8;
 
 /**
  * codes des erreurs d'interprtation
  */
 public static final int errMotInconnu=1;
 public static final int errNomDouble=2;
 public static final int errParam=3;
 public static final int errParenth=4;
 public static final int errPolygone=5;
 public static final int errCouleur=6;
 public static final int errObjet=7;
 public static final int errCommande=8;
 
 /**
  * Constructeur
  */
 public ScriptImageo(String S) {
  super(S," (),="+'\n'+'\r');
 }
 
 /**
  * Fonction renvoyant le code correspondant  un mot cl,
  * et 0 si le mot est inconnu.
  */
 public static int codeImageo(String S) {
  if (S==null) return 0;
  else if (S.equals("POINT")) return ObjImageo.iPoint;
  else if (S.equals("SEGMENT")) return ObjImageo.iSegment;
  else if (S.equals("DROITE")) return ObjImageo.iDroite;
  else if (S.equals("DEMIDROITE")) return ObjImageo.iDemiDroite; 
  else if (S.equals("CERCLE")) return ObjImageo.iCercle;
  else if (S.equals("POINTSUR")) return ObjImageo.iPointSur;
  else if (S.equals("MILIEU")) return ObjImageo.iMilieu;
  else if (S.equals("PARALLELE")) return ObjImageo.iParallele;
  else if (S.equals("PERPENDICULAIRE")) return ObjImageo.iPerpendiculaire;
  else if (S.equals("SYMETRIQUE")) return ObjImageo.iSymetrique;
  else if (S.equals("INTERSECTION")) return ObjImageo.iIntersection;
  else if (S.equals("ROTATION")) return ObjImageo.iRotation;
  else if (S.equals("PROJECTION")) return ObjImageo.iProjection;
  else if (S.equals("TRANSLATION")) return ObjImageo.iTranslation;
  else if (S.equals("HOMOTHETIE")) return ObjImageo.iHomothetie;
  else if (S.equals("MEDIATRICE")) return ObjImageo.iMediatrice;
  else if (S.equals("BISSECTRICE")) return ObjImageo.iBissectrice;
  else if (S.equals("CACHER")) return oCacher;
  else if (S.equals("FIXER")) return oFixer;
  else if (S.equals("CHOISIR")) return oChoisir;
  else if (S.equals("POLYGONE")) return oPolygone;
  else if (S.equals("MESURER")) return ObjImageo.iMesure;
  else if (S.equals("TRACE")) return oTrace;
  else if (S.equals("EPAIS")) return oEpais;
  else if (S.equals("POINTILLE")) return oPointille;
  else if (S.equals("MIXTE")) return oMixte;
  else return 0;
 }
 
 /**
  * Renvoie la liste des paramtres entre parenthses situs aprs une commande 
  */
 public Vector lireParams(FigureImageo fig) {
  Vector rep=new Vector(6);
  String S="";
  ObjImageo oi=null;
  Double d=null;
  boolean fini=false;
  //y a-t-il des paramtres ?
  if (suivant=='(') {
   //passer la parenthse
   lireSuivant();
   passeEspaces();
   while (!fini && codeErreur==0) {
    // est-ce un nombre ?
    try {
     d=new Double(motCourant);
    } catch (NumberFormatException e) {
     d=null;
    } 
    if (d!=null) {
     rep.addElement(d);
     S=S+"N";
     lireSuivant();
     passeEspaces();
    }
    else {
     //est-ce un objet dj cr ?
     oi=fig.trouveObj(motCourant.toUpperCase());
     if (oi!=null) {
      lireSuivant();
      passeEspaces();
     }
     else {
      //est-ce un objet  crer
      int c=codeImageo(motCourant.toUpperCase());
      if (c==0 || c>=oCommande) marqueErreur(errParam);
      else {
       //passer la commande
       lireSuivant();
       passeEspaces();
       //lire les paramtres
       Vector V=lireParams(fig);
       //crer le nouvel objet
       if (codeErreur==0) oi=faireObj(fig,c,V);
       if (oi!=null) {
        oi.stVisible=false;
        fig.addElement(oi);
       }
      } 
     }
     if (oi!=null) {
      //ajouter l'objet
      rep.addElement(oi);
      switch (oi.donneType()) {
       case ObjImageo.iPoint : 
        S=S+"P";
        break;
       case ObjImageo.iSegment :
       case ObjImageo.iDroite :
        S=S+"D";
        break;
       case ObjImageo.iCercle :
        S=S+"C";
        break;
       default : 
        S=S+"I"; 
      }
     }
    } 
    //lire sparateur
    if (suivant!=',' && suivant!=')') marqueErreur(errParenth); 
    else {
     if (suivant==')') fini=true;
     // lire mot suivant
     lireSuivant();
     passeEspaces();
    }
   }
  }
  if (codeErreur==0) rep.addElement(S); // ajouter la chane de dfinition
  else rep=null;
  return rep;
 }
 
 void fairePolygone(FigureImageo fig, Vector V) {
  //ajout d'un polygone sous la forme d'une suite de segments
  jkb.imageo.Point P0,P1,P2;
  String S=(String)V.elementAt(V.size()-1);
  boolean ok=true;
  for (int i=0; i<V.size()-1; i++) {
   if (S.charAt(i)!='P') ok=false;
  }
  if (S.length()<2) ok=false;
  if (!ok) marqueErreur(errPolygone);
  else {
   P0=(jkb.imageo.Point)V.elementAt(0);
   P1=P0;
   for (int k=1; k<V.size()-1; k++) {
    P2=(jkb.imageo.Point)V.elementAt(k);
    fig.addElement(new Segment(P1,P2));
    P1=P2;
   }
   fig.addElement(new Segment(P1,P0));
  }
 }
 
 void execCommande(int com, Vector V) {
  //excution des commandes
  ObjImageo oi;
  String S=(String)V.elementAt(V.size()-1);
  int i=0;
  while (i<V.size()-1 && codeErreur==0) {
   if (S.charAt(i)!='N') {
    oi=(ObjImageo)V.elementAt(i);
    switch (com) {
     case oCacher :
      oi.stVisible=false;
      break;
     case oChoisir :
      oi.stChoisi=true;
      break;
     case oFixer :
      oi.stMobile=false;
      break;
     case oTrace :
      oi.stTrace=true;
      break;
     case oPointille :
      oi.stPointille=true;
      break;
     case oEpais :
      oi.stEpais=true;
      break;
    }
   }
   else marqueErreur(errCommande);
   i++;
  }
 } 
 
 /**
  *Construction d'un objet gomtrique
  */
 public ObjImageo faireObj(FigureImageo fig, int commande, Vector V) {
  // construction d'un objet
  ObjImageo obj=null;
  String S=null;
  Vector VV=null;
  switch (commande) {
   case ObjImageo.iPoint :
    obj=new Point(V);
    break;
   case ObjImageo.iSegment :
    obj=new Segment(V);
    break;
   case ObjImageo.iDroite :
   case ObjImageo.iParallele :
   case ObjImageo.iPerpendiculaire : 
    obj=new Droite(V,commande);
    break;
   case ObjImageo.iMilieu :
   case ObjImageo.iSymetrique :
   case ObjImageo.iTranslation :
   case ObjImageo.iRotation :
   case ObjImageo.iHomothetie :
   case ObjImageo.iProjection :
    obj=new PointConstruit(V,commande);
    break;
   case ObjImageo.iCercle :
    obj=new Cercle(V);
    break;
   case ObjImageo.iIntersection :
    obj=new PointIntersection(V);
    break;
   case ObjImageo.iPointSur :
    obj=new PointSur(V);
    break;
   case ObjImageo.iMesure :
    obj=new Mesure(V);
    break;
   case ObjImageo.iMediatrice :
    S=(String)V.elementAt(V.size()-1);
    if ("PP".equals(S)) {
     VV=new Vector(3);
     Droite D=new Droite(V,ObjImageo.iDroite);
     D.stVisible=false;
     fig.addElement(D);
     VV.addElement(D);
     PointConstruit P=new PointConstruit(V,ObjImageo.iMilieu);
     P.stVisible=false;
     fig.addElement(P);
     VV.addElement(P);
     VV.addElement("DP");
     obj=new Droite(VV,ObjImageo.iPerpendiculaire);
    }
    else marqueErreur(errParam);
    break;
   case ObjImageo.iBissectrice :
    S=(String)V.elementAt(V.size()-1);
    if ("PPP".equals(S)) {
     jkb.imageo.Point P1=(jkb.imageo.Point)V.elementAt(0);
     jkb.imageo.Point P2=(jkb.imageo.Point)V.elementAt(1);
     jkb.imageo.Point P3=(jkb.imageo.Point)V.elementAt(2);
     VV=new Vector();
     VV.addElement(P2);
     VV.addElement(P1);
     VV.addElement("PP");
     Droite D1=new Droite(VV,ObjImageo.iDroite);
     D1.stVisible=false;
     fig.addElement(D1);
     VV.setElementAt(P3,1); 
     Droite D2=new Droite(VV,ObjImageo.iDroite);
     D2.stVisible=false;
     fig.addElement(D2);
     VV.setElementAt(new Double(10),1);
     VV.setElementAt("PN",2);
     Cercle C=new Cercle(VV);
     C.stVisible=false;
     fig.addElement(C);
     VV.removeAllElements();
     VV.addElement(D1);
     VV.addElement(C);
     VV.addElement(new Double(1.0));
     VV.addElement("DCN");
     PointIntersection M1=new PointIntersection(VV);
     M1.stVisible=false;
     fig.addElement(M1);
     VV.setElementAt(D2,0);
     PointIntersection M2=new PointIntersection(VV);
     M2.stVisible=false;
     fig.addElement(M2);
     VV.removeAllElements();
     VV.addElement(M1);
     VV.addElement(M2);
     VV.addElement("PP");
     Droite D3=new Droite(VV,ObjImageo.iDroite);
     D3.stVisible=false;
     fig.addElement(D3);
     VV.removeAllElements();
     VV.addElement(D3);
     VV.addElement(P2);
     VV.addElement("DP");
     obj=new Droite(VV,ObjImageo.iPerpendiculaire);
    }
    else marqueErreur(errParam);
    break;
   default : 
    marqueErreur(errObjet); 
  }
  if (obj!=null) {
   if (!obj.stInitOk) marqueErreur(errParam);
  } 
  if (codeErreur!=0) obj=null;
  return obj;
 } 


 /**
  * Excution de l'instruction suivante
  */
 public void execInstruction(FigureImageo fig) {
  int commande;
  String LeNom, Str1;
  ObjImageo oi;
  //rcuprer le nom de l'objet  crer
  Str1=motCourant.toUpperCase();
  lireSuivant();
  passeEspaces();
  if (suivant=='=') {
   LeNom=Str1;
   //passer le signe =
   lireSuivant();
   passeEspaces();
   Str1=motCourant.toUpperCase();
   lireSuivant();
   passeEspaces();
  }
  else LeNom="";
  if (LeNom.startsWith("COULEUR")) {
   // dfinition d'une couleur
   if (LeNom.equals("COULEURFOND")) fig.couleurFond=Couleurs.donneCouleur(Str1);
   else if (LeNom.equals("COULEURDESSIN")) fig.couleurDessin=Couleurs.donneCouleur(Str1);
   else if (LeNom.equals("COULEURMOBILE")) fig.couleurMobile=Couleurs.donneCouleur(Str1);
   else if (LeNom.equals("COULEURCHOISI")) fig.couleurChoisi=Couleurs.donneCouleur(Str1);
   else if (LeNom.equals("COULEURTEXTE")) fig.couleurTexte=Couleurs.donneCouleur(Str1);
   else if (LeNom.equals("COULEURTRACE")) fig.couleurTrace=Couleurs.donneCouleur(Str1);
   else marqueErreur(errCouleur);
  }
  else if (LeNom.equals("TAILLEFONTE")) {
	fig.tailleFonte=Integer.parseInt(Str1);
  }
  else {
   // dtermination de la commande
   commande=codeImageo(Str1);
   if (commande==0) marqueErreur(errMotInconnu);
   else {
    Vector V=lireParams(fig);
    if (commande>oCommande && codeErreur==0) {
     if (commande==oPolygone) fairePolygone(fig,V);
     else execCommande(commande,V);
    } 
    else if (codeErreur==0) {
     oi=faireObj(fig,commande,V);
     if (oi!=null) {
      if (!fig.ajouteObj(LeNom,oi)) marqueErreur(errNomDouble);
     } 
    } 
   } 
  } 
 }
 
 /**
  * Compilation d'une chane imageo.
  */
 public FigureImageo compile() {
  //interprtation d'une chaine Imageo
  FigureImageo fig=new FigureImageo();
  while (suivant!=carFin && codeErreur==0) {
   execInstruction(fig);
  }
  return fig;
 }
 
 public String messageErreur() {
  String S="";
  switch (codeErreur) {
   case errMotInconnu :
    S="Mot inconnu."+posErreur();
    break;
   case errNomDouble :
    S="Nom dj utilis."+posErreur();
    break;
   case errParam :
    S="Paramtre incorrect."+posErreur();
    break;
   case errParenth :
    S="Parenthse attendue."+posErreur();
    break;
   case errPolygone :
    S="Polygone mal dfini."+posErreur();
    break;
   case errCouleur :
    S="Mauvais nom de couleur."+posErreur();
    break;
   case errObjet :
    S="Code objet non gr."+posErreur();
    break;
   case errCommande :
    S="Code commande non gr."+posErreur();
    break;
   default :
    S=super.messageErreur();
  }
  return S;
 }
 
}

