/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * Logo2d.java
 * JDK 1.1  
 * Version 1.1 Rvision 03/06/2000
 * Partie du package jkb.geom permettant d'utiliser une tortue
 * Logo.
 *
 */

package jkb.mlogo;

import java.awt.*;
import java.util.*;
import jkb.calkul.MathNode;
import jkb.utils.DicoObj;
import jkb.utils.Couleurs;
import jkb.geo3d.*;

/**
 * Classe reprsentant une tortue Logo capable de dessiner dans un
 * plan.
 */
public class Logo3d extends LogoExe implements Dessinateur {

    // codes des commandes prdfinies
    public final static int lgTortue=LogoCom.lgCom+10;
    public final static int lgAV=lgTortue+1;
    public final static int lgRE=lgTortue+2;
    public final static int lgTD=lgTortue+3;
    public final static int lgTG=lgTortue+4;
    public final static int lgID=lgTortue+5;
    public final static int lgIG=lgTortue+6;
    public final static int lgPQ=lgTortue+7;
    public final static int lgCA=lgTortue+8;
    public final static int lgBC=lgTortue+9;
    public final static int lgLC=lgTortue+10;
    public final static int lgFCC=lgTortue+11;
    public final static int lgFPOS=lgTortue+12;

    // position tortue
    double XT, YT, ZT;
    // matrice direction tortue
    double c11, c12, c13, c21, c22, c23, c31,c32,c33;
    // trace tortue
    int trace=1;
    public Color couleurTrace;
    public Color couleurFond;
    // univers tortue
    public Espace E;
    // image contenant la figure utilise pour le double buffering
    Graphics tcg;
    // image programme  excuter
    public String progStr;
    public Vector logoProg;

    /**
     * Constructeur
     */
    public Logo3d(Espace es) {
	super();
	E=es;
	E.D=this;
	E.utiliseImage=true;
	progStr="";
	initialise();
	initDicoTortue();
	modeAngle=MathNode.mDegre;
	tcg=null;
    }

    /**
     * Initialisation position et direction
     */
    public void initialise() {
	XT=0.0; YT=0.0; ZT=0.0;
	c11=0; c12=1; c13=0; 
	c21=-1; c22=0; c23=0; 
	c31=0; c32=0; c33=1;
	couleurTrace=Color.black;
	couleurFond=Color.white;
	trace=1;
    }

    /**
     * Prparation de l'excution
     */
    public void avantExec() {
	initialise();
	tcg=E.img.getGraphics();
	if (tcg!=null) {
	    tcg.setColor(couleurFond);
	    tcg.fillRect(0,0,E.R.largeur,E.R.hauteur);
	    tcg.setColor(couleurTrace);
	    E.traceVers(tcg,0.0,0.0,0.0,0);
	}
    }
  
    /**
     * Fin de l'excution
     */
    public void finirExec() {
	if (tcg!=null) {
	    tcg.dispose();
	    tcg=null;
	} 
    }
 
    /**
     * Enregistrement des commandes Logo
     */
    void initDicoTortue() {
	dico.put("AV",new DicoObj(lgAV,"N"));
	dico.put("RE",new DicoObj(lgRE,"N"));
	dico.put("TD",new DicoObj(lgTD,"N"));
	dico.put("TG",new DicoObj(lgTG,"N"));
	dico.put("ID",new DicoObj(lgID,"N"));
	dico.put("IG",new DicoObj(lgIG,"N"));
	dico.put("PQ",new DicoObj(lgPQ,"N"));
	dico.put("CA",new DicoObj(lgCA,"N"));
	dico.put("BC",new DicoObj(lgBC,null));
	dico.put("LC",new DicoObj(lgLC,null));
	dico.put("FPOS",new DicoObj(lgFPOS,"NNN"));
	dico.put("FCC",new DicoObj(lgFCC,"M"));
    }

    /**
     * Actions tortue
     */
    public void av(double d) {
	XT=XT+d*c11;
	YT=YT+d*c12;
	ZT=ZT+d*c13;
    }
	
    public void re(double d) {
	av(-d);
    }
	
    public void td(double a) {
	double cs = Math.cos(a*Math.PI/180);
	double sn = Math.sin(a*Math.PI/180);
	double b11=((c11*cs)-(c21*sn));
	double b12=((c12*cs)-(c22*sn));
	double b13=((c13*cs)-(c23*sn));
	c21=((c21*cs)+(c11*sn));
	c22=((c22*cs)+(c12*sn));
	c23=((c23*cs)+(c13*sn));
	c11=b11;
	c12=b12;
	c13=b13;
    }
	
    public void tg(double a) {
	td(-a);
    }
	
    public void ig(double a) {
	double cs = Math.cos(a*Math.PI/180);
	double sn = Math.sin(a*Math.PI/180);
	double b21=((c21*cs)-(c31*sn));
	double b22=((c22*cs)-(c32*sn));
	double b23=((c23*cs)-(c33*sn));
	c31=((c31*cs)+(c21*sn));
	c32=((c32*cs)+(c22*sn));
	c33=((c33*cs)+(c23*sn));
	c21=b21;
	c22=b22;
	c23=b23;
    }

    public void id(double a) {
	ig(-a);
    }

    public void pq(double a) {
	double cs = Math.cos(a*Math.PI/180);
	double sn = Math.sin(a*Math.PI/180);
	double b11=((c11*cs)-(c31*sn));
	double b12=((c12*cs)-(c32*sn));
	double b13=((c13*cs)-(c33*sn));
	c31=((c31*cs)+(c11*sn));
	c32=((c32*cs)+(c12*sn));
	c33=((c33*cs)+(c13*sn));
	c11=b11;
	c12=b12;
	c13=b13;
    }

    public void ca(double a) {
	pq(-a);
    }

    public void affiche(String S) {
	if (zoneTexte!=null) zoneTexte.append(S);
    } 

    /**
     * Excution des commandes de la tortue
     */
    public void execComSup(LogoCom c) {
	double d;
	String mot;
	switch (c.oType) {
	case lgAV :
	    d=c.lireNombre(0,modeAngle);
	    av(d);
	    tcg.setColor(couleurTrace);
	    if (tcg!=null) E.traceVers(tcg,XT,YT,ZT,trace);    
	    break;
	case lgRE :
	    d=c.lireNombre(0,modeAngle);
	    re(d);
	    tcg.setColor(couleurTrace);
	    if (tcg!=null) E.traceVers(tcg,XT,YT,ZT,trace);    
	    break;
	case lgTD :
	    d=c.lireNombre(0,modeAngle);
	    td(d);
	    break;
	case lgTG :
	    d=c.lireNombre(0,modeAngle);
	    tg(d);
	    break;
	case lgID :
	    d=c.lireNombre(0,modeAngle);
	    id(d);
	    break;
	case lgIG :
	    d=c.lireNombre(0,modeAngle);
	    ig(d);
	    break;
	case lgPQ :
	    d=c.lireNombre(0,modeAngle);
	    pq(d);
	    break;
	case lgCA :
	    d=c.lireNombre(0,modeAngle);
	    ca(d);
	    break;
	case lgFPOS :
	    XT=c.lireNombre(0,modeAngle);
	    YT=c.lireNombre(1,modeAngle);
	    ZT=c.lireNombre(2,modeAngle);
	    tcg.setColor(couleurTrace);
	    E.traceVers(tcg,XT,YT,ZT,trace);
	    break;
	case lgBC :
	    trace=1;
	    break;
	case lgLC :
	    trace=0;
	    break;
	case lgFCC :
	    mot=c.lireChaine(0);
	    couleurTrace=Couleurs.donneCouleur(mot);
	    break;
	}
    }

    public void dessineFig(Graphics g) {
	if (progStr.length()>0) {
	    LogoStr MS=new LogoStr(progStr);
	    logoProg=MS.compile(dico);
	    if (MS.codeErreur==0) {
		execProg(logoProg,true);
	    }
	    g.drawImage(E.img,0,0,E);
	}
    }
    
}

