/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * Logo2d.java
 * JDK 1.1  
 * Version 1.1 Rvision 03/06/2000
 * Partie du package jkb.geom permettant d'utiliser une tortue
 * Logo.
 *
 */

package jkb.mlogo;

import java.awt.Graphics;
import java.awt.Color;
import java.util.Hashtable;
import jkb.calkul.MathNode;
import jkb.utils.DicoObj;
import jkb.utils.Couleurs;
import jkb.geom.Plan;

/**
 * Classe reprsentant une tortue Logo capable de dessiner dans un
 * plan.
 */
public class Logo2d extends LogoExe {

 // codes des commandes prdfinies
 public final static int lgTortue=LogoCom.lgCom+10;
 public final static int lgAV=lgTortue+1;
 public final static int lgRE=lgTortue+2;
 public final static int lgTD=lgTortue+3;
 public final static int lgTG=lgTortue+4;
 public final static int lgBC=lgTortue+5;
 public final static int lgLC=lgTortue+6;
 public final static int lgFPOS=lgTortue+7;
 public final static int lgFCAP=lgTortue+8;
 public final static int lgFCC=lgTortue+9;
 public final static int lgFCF=lgTortue+10;

 double XT;
 double YT;
 double Cap;
 int Trace=1;
 public Color couleurTrace;
 public Color couleurFond;
 public Plan P;
 private Graphics tcg;
 
 /**
  * Constructeur
  */
 public Logo2d(Plan pl) {
  super();
  P=pl;
  tcg=null;
  couleurTrace=Color.black;
  couleurFond=Color.white;
  Trace=1;
  XT=0.0;
  YT=0.0;
  Cap=0.0;
  initDicoTortue();
  modeAngle=MathNode.mDegre;
 }
 
 /**
  * Prparation de l'excution
  */
 public void avantExec() {
  XT=0.0; YT=0.0; Cap=0.0; Trace=1;
  couleurTrace=Color.black;
  couleurFond=Color.white;
  P.fenetreOrthoCentre(1.0/P.unPixelX());
  finirExec();
  tcg=P.donneCG();
  tcg.setColor(couleurTrace);
  P.couleurFond=couleurFond;
  P.dessineFond(tcg);
  P.traceVers(tcg,XT,YT,0);
 }
  
 /**
  * Fin de l'excution
  */
 public void finirExec() {
  if (tcg!=null) {
   tcg.dispose();
   tcg=null;
  } 
 }
 
 /**
  * Enregistrement des commandes Logo
  */
 void initDicoTortue() {
  dico.put("AV",new DicoObj(lgAV,"N"));
  dico.put("RE",new DicoObj(lgRE,"N"));
  dico.put("TD",new DicoObj(lgTD,"N"));
  dico.put("TG",new DicoObj(lgTG,"N"));
  dico.put("BC",new DicoObj(lgBC,null));
  dico.put("LC",new DicoObj(lgLC,null));
  dico.put("FPOS",new DicoObj(lgFPOS,"NN"));
  dico.put("FCAP",new DicoObj(lgFCAP,"N"));
  dico.put("FCF",new DicoObj(lgFCF,"M"));
  dico.put("FCC",new DicoObj(lgFCC,"M"));
 }
 
 void normaliseCap() {
  while (Cap<0) Cap=Cap+360.0;
  while (Cap>360) Cap=Cap-360.0;
 }
 
 /**
  * Excution des commandes de la tortue
  */
 public void execComSup(LogoCom c) {
  double d;
  String mot;
  switch (c.oType) {
   case lgAV :
    d=c.lireNombre(0,modeAngle);
    XT=XT+d*Math.cos(Cap*Math.PI/180.0);
    YT=YT+d*Math.sin(Cap*Math.PI/180.0);
    tcg.setColor(couleurTrace);
    P.traceVers(tcg,XT,YT,Trace);    
    break;
   case lgTD :
    Cap=Cap-c.lireNombre(0,modeAngle);
    normaliseCap();
    break;
   case lgTG :
    Cap=Cap+c.lireNombre(0,modeAngle);
    normaliseCap();
    break;
   case lgFPOS :
    XT=c.lireNombre(0,modeAngle);
    YT=c.lireNombre(1,modeAngle);
    tcg.setColor(couleurTrace);
    P.traceVers(tcg,XT,YT,Trace);
    break;
   case lgFCAP :
    Cap=c.lireNombre(0,modeAngle);
    break;
   case lgBC :
    Trace=1;
    break;
   case lgLC :
    Trace=0;
    break;
   case lgRE :
    d=c.lireNombre(0,modeAngle);
    XT=XT-d*Math.cos(Cap*Math.PI/180.0);
    YT=YT-d*Math.sin(Cap*Math.PI/180.0);
    tcg.setColor(couleurTrace);
    P.traceVers(tcg,XT,YT,Trace);    
    break;
   case lgFCC :
    mot=c.lireChaine(0);
    couleurTrace=Couleurs.donneCouleur(mot);
    break;
   case lgFCF :
    mot=c.lireChaine(0);
    couleurFond=Couleurs.donneCouleur(mot);
    P.couleurFond=couleurFond;
    P.dessineFond(tcg);
    break;
  }
 }
 
}

