/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * LogoExe.java
 * JDK 1.1  
 * Version 1.1 Rvision 03/06/2000
 * Classe du package jkb.calkul reprsentant un objet pouvant
 * excuter un programme MiniLogo.
 *
 */

package jkb.mlogo;

import java.util.Vector;
import java.util.Hashtable;
import java.awt.TextArea;
import jkb.calkul.MathNode;
import jkb.utils.Div;

/**
 * Classe reprsentant un objet pouvant excuter un programme
 * MiniLogo.
 * Gre les commandes DANS, POUR, SI, TANTQUE, REPETE, ....
 */
public class LogoExe {

 public final static String MLogoVersion = "MiniLogo v1.0a (c) 2000 Bruno K.";
 public int modeAngle=MathNode.mRadian;
 
 /**
  * Le dictionnaire MiniLogo : contient les noms de commandes,
  * les variables, les procdures dfinies par l'utilisateur.
  */
 public Hashtable dico;
 
 /**
  * Zone d'affichage des rsultats
  */
 public TextArea zoneTexte;
 
 /**
  * Constructeur; cre le dictionnaire de base
  */
 public LogoExe() {
  dico=new Hashtable();
  MathNode.initDico(dico);
  LogoCom.initDicoLogo(dico);
  zoneTexte=null;
 }

 public void defZoneTexte(TextArea t) {
  zoneTexte=t;
 }
 
 public void affiche(String S) {
  if (zoneTexte!=null) zoneTexte.append(S);
 }

 /**
  * Prparation de l'excution
  */
 public void avantExec() {
 }
 
 /**
  * Aprs l'excution
  */
 public void finirExec() {
 }
 
 /**
  * Excution du programme contenu dans V, tableau d'objets du type
  * LogoCom.
  */
 public void execProg(Vector V, boolean prem) {
  LogoCom c;
  if (prem) avantExec();
  for (int i=0; i<V.size(); i++) {
   c=(LogoCom)V.elementAt(i);
   if (c.oType>LogoCom.lgCom && c.oType<LogoCom.lgMaxCom) execCom(c);
   else execComSup(c);
  }
  if (prem) finirExec();
 }
 
 /**
  * Commandes ajoutes.
  */
 public synchronized void execComSup(LogoCom c) {
 }

 /**
  * Procdure d'erreur
  */
 public void procErreur(LogoCom c) {
  affiche("Commande inconnue.");
 }
  
 /**
  * Excution du programme contenu dans V, tableau d'objets du type
  * LogoCom.
  */
 public void execCom(LogoCom c) {
  double d;
  Vector vv;
  String nom;
  switch (c.oType) {
   case LogoCom.lgDANS :
    nom=c.lireChaine(0);
    d=c.lireNombre(1,modeAngle);
    MathNode.defVar(nom,d,dico); 
    break;
   case LogoCom.lgREPETE :
    Double DD=new Double(c.lireNombre(0,modeAngle));
    int n=DD.intValue();
    vv=c.lireProg(1);
    for (int i=0; i<n; i++) execProg(vv,false);
    break;
   case LogoCom.lgSI :
    d=c.lireNombre(0,modeAngle);
    if (d==1.0) {
     vv=c.lireProg(1);
     execProg(vv,false);
    }
    break;
   case LogoCom.lgTANTQUE :
    d=c.lireNombre(0,modeAngle);
    vv=c.lireProg(1);
    while (d==1.0) {
     execProg(vv,false);
     d=c.lireNombre(0,modeAngle);
    }
    break;
   case LogoCom.lgPROG : 
    execProg((Vector)c.obj,false);
    break;
   case LogoCom.lgAFF :
    affiche(c.lireChaine(0));
    break;
   case LogoCom.lgAFFN :
    d=c.lireNombre(0,modeAngle);
    affiche(Div.DoubleToStr(d,8));
    break;
   case LogoCom.lgRET :
    affiche(""+'\n');
    break;    
   default : 
    procErreur(c);
  }
 }
 
}

