/*
 * Copyright (c) 1999-2000 Kostrzewa Bruno
 *
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for NON-COMMERCIAL purposes and without fee is hereby
 * granted provided that this copyright notice appears in all copies.
 * 
 * ComImageo.java
 * JDK 1.1  
 * Version 1.1 Rvision 8/7/2000
 * Partie du package utils permettant d'analyser une chane de
 * caractres.
 *
 */

package jkb.utils;

import java.util.*;

/**
 * Classe reprsentant une chane de caractres  analyser.
 */
public class ComStr extends StringTokenizer {
 
 public final static char carFin='\0';

 /**
  * Code erreur. 0 si pas d'erreur.
  */
 public int codeErreur;
 
 public char suivant;
 public String motCourant;
 public int numLigne;
 public int numColonne;
 
 /**
  * Constructeur avec sparateurs par dfaut
  */
 public ComStr(String S) {
  super(S);
  codeErreur=0;
  numColonne=0;
  numLigne=0;
  lireSuivant();
 }
 
 /**
  * Constructeur dfinissant les sparateus  utiliser.
  */
 public ComStr(String S, String sep) {
  super(S,sep,true);
  codeErreur=0;
  numColonne=0;
  numLigne=0;
  lireSuivant();
 }

 /**
  * Marquage des erreurs
  */
 public void marqueErreur(int n) {
  if (n==0) codeErreur=0;
  else if (codeErreur==0) codeErreur=n;
 }
 
 /**
  * Fonction renvoyant la position sous forme de chane.
  */
 public final String posErreur() {
  numLigne++;
  numColonne++;
  String S=" ("+numLigne+","+numColonne+") ";
  numLigne--;
  numColonne--;
  return S;
 }
 
 /**
  * Fonction renvoyant un message d'erreur avec entre
  * parenthses la position de l'erreur.
  */
 public String messageErreur() {
  if (codeErreur==0) return "OK";
  else return "Erreur "+codeErreur+posErreur();
 }
 
 /**
  * Lecture de l'lment suivant qui est plac dans la variable
  * motCourant; la variable suivant contient le premier caractre
  * de motCourant.
  */
 public void lireSuivant() {
  if (hasMoreTokens()) {
   motCourant=nextToken();
   suivant=motCourant.charAt(0);
   if (suivant=='\n') {
    numLigne++;
    numColonne=0;
   }
   else numColonne=numColonne+motCourant.length();
  } 
  else {
   motCourant="";
   suivant=carFin;
  }  
 }

 /**
  * Passer les espaces et les sauts de ligne
  */
 public void passeEspaces() {
  while (suivant==' ' || suivant=='\r' || suivant=='\n') 
   lireSuivant();
 }
 
 /**
  * Lecture d'une chaine de caractres entre guillemets.
  */
 public String lireChaine() {
  passeEspaces();
  String S="";
  char c=suivant;
  if (c=='"') {
   // on passe les guillemets
   lireSuivant();
   while (suivant!='"' && suivant!='\n' && suivant!=carFin) {
    S=S+motCourant;
    lireSuivant();
   }
   // on passe les guillemets
   lireSuivant();
   passeEspaces();
  }
  return S;
 }
  
}  

